@@ -1,10 +1,34 @@
+/* Copyright (c) 2002,2003,2004,2009 James M. Kretchmar
+ *
+ * This file is part of Owl.
+ *
+ * Owl is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Owl is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Owl.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * ---------------------------------------------------------------
+ * 
+ * As of Owl version 2.1.12 there are patches contributed by
+ * developers of the branched BarnOwl project, Copyright (c)
+ * 2006-2009 The BarnOwl Developers. All rights reserved.
+ */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: commands.c,v 1.76 2004/04/10 14:47:41 kretch Exp $";
+static const char fileIdent[] = "$Id: commands.c,v 1.80 2009/03/29 18:58:21 kretch Exp $";
 
 /* fn is "char *foo(int argc, char **argv, char *buff)" */
 #define OWLCMD_ARGS(name, fn, ctx, summary, usage, description) \
@@ -131,8 +155,10 @@ owl_cmd commands_to_init[]
 
   OWLCMD_ARGS("aimwrite", owl_command_aimwrite, OWL_CTX_INTERACTIVE,
 	      "send an AIM message",
-	      "aimzwrite <user>",
-	      "Send an aim message to a user.\n"),
+	      "aimwrite <user> [-m <message...>]",
+	      "Send an aim message to a user.\n\n" 
+              "The following options are available:\n\n"
+              "-m    Specifies a message to send without prompting.\n"),
 
   OWLCMD_ARGS("loopwrite", owl_command_loopwrite, OWL_CTX_INTERACTIVE,
 	      "send a loopback message",
@@ -442,13 +468,6 @@ owl_cmd commands_to_init[]
   OWLCMD_VOID("pop-message", owl_command_pop_message, OWL_CTX_RECWIN,
 	      "pops up a message in a window", "", ""),
 
-  OWLCMD_VOID("openurl", owl_command_openurl, OWL_CTX_INTERACTIVE,
-	      "opens up a URL from the current message",
-	      "", 
-	      "Uses the 'webbrowser' variable to determine\n"
-	      "which browser to use.  Currently, 'netscape'\n"
-	      "and 'galeon' are supported.\n"),
-
   OWLCMD_ARGS("zaway", owl_command_zaway, OWL_CTX_INTERACTIVE,
 	      "Set, enable or disable zephyr away message",
 	      "zaway [ on | off | toggle ]\n"
@@ -703,6 +722,10 @@ owl_cmd commands_to_init[]
 	      "returns the value of a variable",
 	      "getvar <varname>", ""),
 
+  OWLCMD_ARGS("getstyle", owl_command_getstyle, OWL_CTX_INTERACTIVE,
+	      "returns the name of the style for the current view",
+	      "", ""),
+
   OWLCMD_ARGS("search", owl_command_search, OWL_CTX_INTERACTIVE,
 	      "search messages for a particular string",
 	      "search [-r] [<string>]",
@@ -997,10 +1020,7 @@ void owl_command_about()
 
 void owl_command_version()
 {
-  char buff[1024];
-
-  sprintf(buff, "Owl version %s", OWL_VERSION_STRING);
-  owl_function_makemsg(buff);
+  owl_function_makemsg("Owl version %s", OWL_VERSION_STRING);
 }
 
 char *owl_command_aim(int argc, char **argv, char *buff)
@@ -1555,15 +1575,15 @@ char *owl_command_aperl(int argc, char **argv, char *buff)
 
 char *owl_command_multi(int argc, char **argv, char *buff)
 {
-  char *lastrv = NULL, *newbuff;
+  char *lastrv = NULL, *dupbuff, *newbuff;
   char **commands;
   int  ncommands, i;
   if (argc < 2) {
     owl_function_makemsg("Invalid arguments to 'multi' command.");    
     return NULL;
   }
-  newbuff = owl_strdup(buff);
-  newbuff = skiptokens(newbuff, 1);
+  dupbuff = owl_strdup(buff);
+  newbuff = skiptokens(dupbuff, 1);
   if (!strcmp(argv[0], "(")) {
     for (i=strlen(newbuff)-1; i>=0; i--) {
       if (newbuff[i] == ')') {
@@ -1583,6 +1603,7 @@ char *owl_command_multi(int argc, char **argv, char *buff)
     }
     lastrv = owl_function_command(commands[i]);
   }
+  owl_free(dupbuff);
   atokenize_free(commands, ncommands);
   return lastrv;
 }
@@ -1838,8 +1859,9 @@ char *owl_command_zwrite(int argc, char **argv, char *buff)
 
 char *owl_command_aimwrite(int argc, char **argv, char *buff)
 {
-  char *newbuff;
-  int i, j;
+  char *newbuff, *recip, **myargv;
+  int i, j, myargc;
+  owl_message *m;
   
   if (!owl_global_is_aimloggedin(&g)) {
     owl_function_makemsg("You are not logged in to AIM.");
@@ -1851,6 +1873,56 @@ char *owl_command_aimwrite(int argc, char **argv, char *buff)
     return(NULL);
   }
 
+  myargv=argv;
+  if (argc<0) {
+    owl_function_error("Unbalanced quotes in aimwrite");
+    return(NULL);
+  }
+  myargc=argc;
+  if (myargc && *(myargv[0])!='-') {
+    myargc--;
+    myargv++;
+  }
+  while (myargc) {
+    if (!strcmp(myargv[0], "-m")) {
+      if (myargc<2) {
+	break;
+      }
+
+      /* Once we have -m, gobble up everything else on the line */
+      myargv++;
+      myargc--;
+      newbuff=owl_malloc(1);
+      newbuff=owl_strdup("");
+      while (myargc) {
+	newbuff=realloc(newbuff, strlen(newbuff)+strlen(myargv[0])+5);
+	strcat(newbuff, myargv[0]);
+	strcat(newbuff, " ");
+	myargc--;
+	myargv++;
+      }
+      newbuff[strlen(newbuff)-1]='\0'; /* remove last space */
+
+      recip=owl_malloc(strlen(argv[0])+5);
+      sprintf(recip, "%s ", argv[1]);
+      owl_aim_send_im(recip, newbuff);
+      m=owl_function_make_outgoing_aim(newbuff, recip);
+      if (m) { 
+          owl_global_messagequeue_addmsg(&g, m);
+      } else {
+          owl_function_error("Could not create outgoing AIM message");
+      }
+
+      owl_free(recip);
+      owl_free(newbuff);
+      return(NULL);
+    } else {
+      /* we don't care */
+      myargv++;
+      myargc--;
+    }
+  }
+
   /* squish arguments together to make one screenname w/o spaces for now */
   newbuff=owl_malloc(strlen(buff)+5);
   sprintf(newbuff, "%s ", argv[0]);
@@ -1975,6 +2047,7 @@ char *owl_command_view(int argc, char **argv, char *buff)
       char *foo;
       foo=owl_function_create_negative_filter(owl_view_get_filtname(owl_global_get_current_view(&g)));
       owl_function_change_currentview_filter(foo);
+      owl_free(foo);
       return(NULL);
     }
   }
@@ -2156,11 +2229,6 @@ void owl_command_pop_message(void)
   owl_function_curmsg_to_popwin();
 }
 
-void owl_command_openurl(void)
-{
-  owl_function_openurl();
-}
-
 char *owl_command_delete(int argc, char **argv, char *buff)
 {
   int move_after = 1;
@@ -2398,6 +2466,18 @@ char *owl_command_aimlogout(int argc, char **argv, char *buff)
   return(NULL);
 }
 
+char *owl_command_getstyle(int argc, char **argv, char *buff)
+{
+  char *stylename;
+  if (argc != 1) {
+    owl_function_makemsg("Wrong number of arguments for %s", argv[0]);
+    return NULL;
+  }
+  stylename = owl_view_get_style_name(owl_global_get_current_view(&g));
+  if (stylename) stylename = owl_strdup(stylename);
+  return stylename;
+}
+
 /*********************************************************************/
 /************************** EDIT SPECIFIC ****************************/
 /*********************************************************************/
@@ -2408,9 +2488,11 @@ void owl_command_edit_cancel(owl_editwin *e)
 
   owl_function_makemsg("Command cancelled.");
 
-  hist=owl_editwin_get_history(e);
-  owl_history_store(hist, owl_editwin_get_text(e));
-  owl_history_reset(hist);
+  if(e->echochar == 0) {
+    hist=owl_editwin_get_history(e);
+    owl_history_store(hist, owl_editwin_get_text(e));
+    owl_history_reset(hist);
+  }
 
   owl_editwin_fullclear(e);
   owl_global_set_needrefresh(&g);
