@@ -1,7 +1,31 @@
+/* Copyright (c) 2002,2003,2004,2009 James M. Kretchmar
+ *
+ * This file is part of Owl.
+ *
+ * Owl is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Owl is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Owl.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * ---------------------------------------------------------------
+ * 
+ * As of Owl version 2.1.12 there are patches contributed by
+ * developers of the branched BarnOwl project, Copyright (c)
+ * 2006-2009 The BarnOwl Developers. All rights reserved.
+ */
+
 #include <string.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: filter.c,v 1.21 2005/01/05 04:04:53 kretch Exp $";
+static const char fileIdent[] = "$Id: filter.c,v 1.24 2009/03/29 18:43:40 kretch Exp $";
 
 #define OWL_FILTER_MAXRECURSE 20
 
@@ -12,7 +36,7 @@ int owl_filter_init_fromstring(owl_filter *f, char *name, char *string)
 
   argv=owl_parseline(string, &argc);
   out=owl_filter_init(f, name, argc, argv);
-  /* owl_parsefree(argv, argc); */
+  owl_parsefree(argv, argc);
   return(out);
 }
 
@@ -451,41 +475,48 @@ int _owl_filter_message_match_recurse(owl_filter *f, owl_message *m, owl_list *f
 
 }
 
-void owl_filter_print(owl_filter *f, char *out)
+char *owl_filter_print(owl_filter *f)
 {
   int i, j;
   owl_filterelement *fe;
-  char *tmp;
-
-  strcpy(out, owl_filter_get_name(f));
-  strcat(out, ": ");
+  GString *out = g_string_new("");
 
   if (f->color!=OWL_COLOR_DEFAULT) {
-    strcat(out, "-c ");
-    strcat(out, owl_util_color_to_string(f->color));
-    strcat(out, " ");
+    g_string_append(out, "-c ");
+    if (f->color < 8) {
+      g_string_append(out, owl_util_color_to_string(f->color));
+    } else {
+      g_string_append_printf(out, "%i",f->color);
+    }
+    g_string_append(out, " ");
   }
 
   j=owl_list_get_size(&(f->fes));
   for (i=0; i<j; i++) {
     fe=owl_list_get_element(&(f->fes), i);
-    tmp=owl_filterelement_to_string(fe);
-    strcat(out, tmp);
-    owl_free(tmp);
+    g_string_append(out, owl_filterelement_to_string(fe));
   }
-  strcat(out, "\n");
+
+  return g_string_free(out, 0);
 }
 
 /* Return 1 if the filters 'a' and 'b' are equivalent, 0 otherwise */
 int owl_filter_equiv(owl_filter *a, owl_filter *b)
 {
-  char buff[LINE], buff2[LINE];
+  char *buffa, *buffb;
+  int ret;
 
-  owl_filter_print(a, buff);
-  owl_filter_print(b, buff2);
+  buffa = owl_filter_print(a);
+  buffb = owl_filter_print(b);
 
-  if (!strcmp(buff, buff2)) return(1);
-  return(0);
+  ret = !strcmp(buffa, buffb);
+  ret = ret && !strcmp(owl_filter_get_name(a),
+                       owl_filter_get_name(b));
+
+  owl_free(buffa);
+  owl_free(buffb);
+
+  return ret;
 }
 
 /* Private
