@@ -1,3 +1,27 @@
+/* Copyright (c) 2002,2003,2004,2009 James M. Kretchmar
+ *
+ * This file is part of Owl.
+ *
+ * Owl is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Owl is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Owl.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * ---------------------------------------------------------------
+ * 
+ * As of Owl version 2.1.12 there are patches contributed by
+ * developers of the branched BarnOwl project, Copyright (c)
+ * 2006-2009 The BarnOwl Developers. All rights reserved.
+ */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -12,7 +36,7 @@
 #include <signal.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: functions.c,v 1.126 2005/01/05 04:04:53 kretch Exp $";
+static const char fileIdent[] = "$Id: functions.c,v 1.141 2009/04/05 23:36:53 kretch Exp $";
 
 void owl_function_noop(void)
 {
@@ -124,42 +148,27 @@ void owl_function_show_license()
 
   text=""
     "Owl version " OWL_VERSION_STRING "\n"
-    "Copyright (c) 2004 James Kretchmar. All rights reserved.\n"
     "\n"
-    "Redistribution and use in source and binary forms, with or without\n"
-    "modification, are permitted provided that the following conditions are\n"
-    "met:\n"
+    "Copyright (c) 2002,2003,2004,2009 James M. Kretchmar\n"
     "\n"
-    "   * Redistributions of source code must retain the above copyright\n"
-    "     notice, this list of conditions and the following disclaimer.\n"
+    "Owl is free software: you can redistribute it and/or modify\n"
+    "it under the terms of the GNU General Public License as published by\n"
+    "the Free Software Foundation, either version 3 of the License, or\n"
+    "(at your option) any later version.\n"
     "\n"
-    "   * Redistributions in binary form must reproduce the above copyright\n"
-    "     notice, this list of conditions and the following disclaimer in\n"
-    "     the documentation and/or other materials provided with the\n"
-    "     distribution.\n"
+    "Owl is distributed in the hope that it will be useful,\n"
+    "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+    "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+    "GNU General Public License for more details.\n"
     "\n"
-    "   * Redistributions in any form must be accompanied by information on\n"
-    "     how to obtain complete source code for the Owl software and any\n"
-    "     accompanying software that uses the Owl software. The source code\n"
-    "     must either be included in the distribution or be available for no\n"
-    "     more than the cost of distribution plus a nominal fee, and must be\n"
-    "     freely redistributable under reasonable conditions. For an\n"
-    "     executable file, complete source code means the source code for\n"
-    "     all modules it contains. It does not include source code for\n"
-    "     modules or files that typically accompany the major components of\n"
-    "     the operating system on which the executable file runs.\n"
+    "You should have received a copy of the GNU General Public License\n"
+    "along with Owl.  If not, see <http://www.gnu.org/licenses/>.\n"
     "\n"
-    "THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR\n"
-    "IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED\n"
-    "WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR\n"
-    "NON-INFRINGEMENT, ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE\n"
-    "LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR\n"
-    "CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF\n"
-    "SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR\n"
-    "BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,\n"
-    "WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE\n"
-    "OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN\n"
-    "IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.\n";
+    "---------------------------------------------------------------\n"
+    "\n"
+    "As of Owl version 2.1.12 there are patches contributed by\n"
+    "developers of the branched BarnOwl project, Copyright (c)\n"
+    "2006-2009 The BarnOwl Developers. All rights reserved.\n";
   owl_function_popless_text(text);
 }
 
@@ -280,7 +289,7 @@ owl_message *owl_function_make_outgoing_loopback(char *body)
 void owl_function_zwrite_setup(char *line)
 {
   owl_editwin *e;
-  char buff[1024];
+  char *buff;
   owl_zwrite z;
   int ret;
 
@@ -310,21 +319,19 @@ void owl_function_zwrite_setup(char *line)
 
   owl_editwin_clear(e);
   owl_editwin_set_dotsend(e);
-  strcpy(buff, "----> ");
-  strcat(buff, line);
-  strcat(buff, "\n");
+  buff=owl_sprintf("----> %s\n", line);
   owl_editwin_set_locktext(e, buff);
 
-  /* make it active */
   owl_global_set_typwin_active(&g);
 
   owl_global_set_buffercommand(&g, line);
+  owl_free(buff);
 }
 
 void owl_function_aimwrite_setup(char *line)
 {
   owl_editwin *e;
-  char buff[1024];
+  char *buff;
 
   /* check the arguments */
 
@@ -340,15 +347,13 @@ void owl_function_aimwrite_setup(char *line)
 
   owl_editwin_clear(e);
   owl_editwin_set_dotsend(e);
-  strcpy(buff, "----> ");
-  strcat(buff, line);
-  strcat(buff, "\n");
+  buff=owl_sprintf("----> %s\n", line);
   owl_editwin_set_locktext(e, buff);
 
-  /* make it active */
   owl_global_set_typwin_active(&g);
 
   owl_global_set_buffercommand(&g, line);
+  owl_free(buff);
 }
 
 void owl_function_loopwrite_setup()
@@ -434,12 +439,21 @@ void owl_function_zcrypt(char *line, char *msg)
 
   mymsg=owl_zwrite_get_message(&z);
 #ifdef OWL_ENABLE_ZCRYPT
-  cryptmsg=owl_malloc(strlen(mymsg)*4);
+  /* Allocate enough space for the crypted message. For each byte of
+   * the message, the encoded cyphertext will have two bytes. Block
+   * size is 8 bytes of input, or 16 bytes of output, so make sure we
+   * have at least one block worth of space allocated. If the message
+   * is empty, no blocks are sent, but we still allocate one
+   * block. The additional 16 bytes also provide space for the null
+   * terminator, as we will never use all of it for cyphertext.
+   */
+  cryptmsg=owl_malloc((strlen(mymsg)*2)+16);
   ret=owl_zcrypt_encrypt(cryptmsg, mymsg, owl_zwrite_get_class(&z), owl_zwrite_get_instance(&z));
   if (ret) {
     owl_function_error("Error in zcrypt, possibly no key found.  Message not sent.");
     owl_function_beep();
     owl_free(cryptmsg);
+    owl_zwrite_free(&z);
     return;
   }
 #else
@@ -875,12 +889,14 @@ void owl_function_unsuball()
 void owl_function_loadsubs(char *file)
 {
   int ret, ret2;
-  char *foo;
+  char *foo, *path;
 
   if (file==NULL) {
     ret=owl_zephyr_loadsubs(NULL, 0);
   } else {
-    ret=owl_zephyr_loadsubs(file, 1);
+    path = owl_util_makepath(file);
+    ret=owl_zephyr_loadsubs(path, 1);
+    free(path);
   }
 
   /* for backwards compatibility for now */
@@ -1010,104 +1026,6 @@ void owl_function_quit()
   exit(0);
 }
 
-void owl_function_openurl()
-{
-  /* visit the first url in the current message */
-  owl_message *m;
-  owl_view *v;
-  char *ptr1, *ptr2, *text, url[LINE], tmpbuff[LINE];
-  int webbrowser;
-
-  webbrowser = owl_global_get_webbrowser(&g);
-
-  if (webbrowser < 0 || webbrowser == OWL_WEBBROWSER_NONE) {
-    owl_function_error("No browser selected");
-    return;
-  }
-
-  v=owl_global_get_current_view(&g);
-  
-  m=owl_view_get_element(v, owl_global_get_curmsg(&g));
-
-  if (!m || owl_view_get_size(v)==0) {
-    owl_function_error("No current message selected");
-    return;
-  }
-
-  text=owl_message_get_text(m);
-
-  /* First look for a good URL */  
-  if ((ptr1=strstr(text, "http://"))!=NULL) {
-    ptr2=strpbrk(ptr1, " \n\t");
-    if (ptr2) {
-      strncpy(url, ptr1, ptr2-ptr1+1);
-      url[ptr2-ptr1+1]='\0';
-    } else {
-      strcpy(url, ptr1);
-    }
-
-    /* if we had <http strip a trailing > */
-    if (ptr1>text && ptr1[-1]=='<') {
-      if (url[strlen(url)-1]=='>') {
-	url[strlen(url)-1]='\0';
-      }
-    }
-  } else if ((ptr1=strstr(text, "https://"))!=NULL) {
-    /* Look for an https URL */  
-    ptr2=strpbrk(ptr1, " \n\t");
-    if (ptr2) {
-      strncpy(url, ptr1, ptr2-ptr1+1);
-      url[ptr2-ptr1+1]='\0';
-    } else {
-      strcpy(url, ptr1);
-    }
-    
-    /* if we had <http strip a trailing > */
-    if (ptr1>text && ptr1[-1]=='<') {
-      if (url[strlen(url)-1]=='>') {
-	url[strlen(url)-1]='\0';
-      }
-    }
-  } else if ((ptr1=strstr(text, "www."))!=NULL) {
-    /* if we can't find a real url look for www.something */
-    ptr2=strpbrk(ptr1, " \n\t");
-    if (ptr2) {
-      strncpy(url, ptr1, ptr2-ptr1+1);
-      url[ptr2-ptr1+1]='\0';
-    } else {
-      strcpy(url, ptr1);
-    }
-  } else {
-    owl_function_beep();
-    owl_function_error("Could not find URL to open.");
-    return;
-  }
-
-  /* Make sure there aren't any quotes or \'s in the url */
-  for (ptr1 = url; *ptr1; ptr1++) {
-    if (*ptr1 == '"' || *ptr1 == '\\') {
-      owl_function_beep();
-      owl_function_error("URL contains invalid characters.");
-      return;
-    }
-  }
-  
-  /* NOTE: There are potentially serious security issues here... */
-
-  /* open the page */
-  owl_function_makemsg("Opening %s", url);
-  if (webbrowser == OWL_WEBBROWSER_NETSCAPE) {
-    snprintf(tmpbuff, LINE, "netscape -remote \"openURL(%s)\" > /dev/null 2> /dev/null", url);
-    system(tmpbuff); 
-  } else if (webbrowser == OWL_WEBBROWSER_GALEON) {
-    snprintf(tmpbuff, LINE, "galeon \"%s\" > /dev/null 2> /dev/null &", url);
-    system(tmpbuff); 
-  } else if (webbrowser == OWL_WEBBROWSER_OPERA) {
-    snprintf(tmpbuff, LINE, "opera \"%s\" > /dev/null 2> /dev/null &", url);
-    system(tmpbuff); 
-  }
-}
-
 void owl_function_calculate_topmsg(int direction)
 {
   int recwinlines, topmsg, curmsg;
@@ -1227,7 +1145,7 @@ int owl_function_calculate_topmsg_paged(int direction, owl_view *v, int curmsg,
 
 int owl_function_calculate_topmsg_normal(int direction, owl_view *v, int curmsg, int topmsg, int recwinlines)
 {
-  int savey, j, i, foo, y;
+  int savey, i, foo, y;
 
   if (curmsg<0) return(topmsg);
     
@@ -1236,17 +1154,23 @@ int owl_function_calculate_topmsg_normal(int direction, owl_view *v, int curmsg,
     topmsg=owl_function_calculate_topmsg_center(direction, v, curmsg, 0, recwinlines);
   }
 
-  /* Find number of lines from top to bottom of curmsg (store in savey) */
-  savey=0;
-  for (i=topmsg; i<=curmsg; i++) {
-    savey+=owl_message_get_numlines(owl_view_get_element(v, i));
+  /* If curmsg is so far past topmsg that there are more messages than
+     lines, skip the line counting that follows because we're
+     certainly off screen.  */
+  savey=curmsg-topmsg;
+  if (savey <= recwinlines) {
+    /* Find number of lines from top to bottom of curmsg (store in savey) */
+    savey = 0;
+    for (i=topmsg; i<=curmsg; i++) {
+      savey+=owl_message_get_numlines(owl_view_get_element(v, i));
+    }
   }
 
   /* If we're off the bottom of the screen, set the topmsg to curmsg
    * and scroll upwards */
   if (savey > recwinlines) {
     topmsg=curmsg;
-    savey=owl_message_get_numlines(owl_view_get_element(v, i));
+    savey=owl_message_get_numlines(owl_view_get_element(v, curmsg));
     direction=OWL_DIRECTION_UPWARDS;
   }
   
@@ -1254,20 +1178,20 @@ int owl_function_calculate_topmsg_normal(int direction, owl_view *v, int curmsg,
   if (direction == OWL_DIRECTION_UPWARDS || direction == OWL_DIRECTION_NONE) {
     if (savey < (recwinlines / 4)) {
       y=0;
-      for (j=curmsg; j>=0; j--) {
-	foo=owl_message_get_numlines(owl_view_get_element(v, j));
+      for (i=curmsg; i>=0; i--) {
+	foo=owl_message_get_numlines(owl_view_get_element(v, i));
 	/* will we run the curmsg off the screen? */
 	if ((foo+y) >= recwinlines) {
-	  j++;
-	  if (j>curmsg) j=curmsg;
+	  i++;
+	  if (i>curmsg) i=curmsg;
 	  break;
 	}
 	/* have saved 1/2 the screen space? */
 	y+=foo;
 	if (y > (recwinlines / 2)) break;
       }
-      if (j<0) j=0;
-      return(j);
+      if (i<0) i=0;
+      return(i);
     }
   }
 
@@ -1276,14 +1200,14 @@ int owl_function_calculate_topmsg_normal(int direction, owl_view *v, int curmsg,
     if (savey > ((recwinlines * 3)/4)) {
       y=0;
       /* count lines from the top until we can save 1/2 the screen size */
-      for (j=topmsg; j<curmsg; j++) {
-	y+=owl_message_get_numlines(owl_view_get_element(v, j));
+      for (i=topmsg; i<curmsg; i++) {
+	y+=owl_message_get_numlines(owl_view_get_element(v, i));
 	if (y > (recwinlines / 2)) break;
       }
-      if (j==curmsg) {
-	j--;
+      if (i==curmsg) {
+	i--;
       }
-      return(j+1);
+      return(i+1);
     }
   }
 
@@ -1321,7 +1245,8 @@ void owl_function_debugmsg(char *fmt, ...)
 {
   FILE *file;
   time_t now;
-  char buff1[LINE], buff2[LINE];
+  char buff1[LINE];
+  char *buff2;
   va_list ap;
   va_start(ap, fmt);
 
@@ -1334,13 +1259,14 @@ void owl_function_debugmsg(char *fmt, ...)
   strcpy(buff1, ctime(&now));
   buff1[strlen(buff1)-1]='\0';
 
-  owl_global_get_runtime_string(&g, buff2);
+  buff2=owl_global_get_runtime_string(&g);
   
   fprintf(file, "[%i -  %s - %s]: ", (int) getpid(), buff1, buff2);
   vfprintf(file, fmt, ap);
   fprintf(file, "\n");
   fclose(file);
-
+  owl_free(buff2);
+  
   va_end(ap);
 }
 
@@ -1463,35 +1389,14 @@ void owl_function_popless_file(char *filename)
 
 void owl_function_about()
 {
-  char buff[5000];
-
-  sprintf(buff, "This is owl version %s\n", OWL_VERSION_STRING);
-  strcat(buff, "\nOwl was written by James Kretchmar at the Massachusetts\n");
-  strcat(buff, "Institute of Technology.  The first version, 0.5, was\n");
-  strcat(buff, "released in March 2002.\n");
-  strcat(buff, "\n");
-  strcat(buff, "The name 'owl' was chosen in reference to the owls in the\n");
-  strcat(buff, "Harry Potter novels, who are tasked with carrying messages\n");
-  strcat(buff, "between Witches and Wizards.\n");
-  strcat(buff, "\n");
-  strcat(buff, "Copyright 2002 Massachusetts Institute of Technology\n");
-  strcat(buff, "\n");
-  strcat(buff, "Permission to use, copy, modify, and distribute this\n");
-  strcat(buff, "software and its documentation for any purpose and without\n");
-  strcat(buff, "fee is hereby granted, provided that the above copyright\n");
-  strcat(buff, "notice and this permission notice appear in all copies\n");
-  strcat(buff, "and in supporting documentation.  No representation is\n");
-  strcat(buff, "made about the suitability of this software for any\n");
-  strcat(buff, "purpose.  It is provided \"as is\" without express\n");
-  strcat(buff, "or implied warranty.\n");
-  owl_function_popless_text(buff);
+  owl_function_show_license();
 }
 
 void owl_function_info()
 {
   owl_message *m;
   owl_fmtext fm, attrfm;
-  char buff[10000];
+  char *buff;
   owl_view *v;
 #ifdef HAVE_LIBZEPHYR
   ZNotice_t *n;
@@ -1508,8 +1413,9 @@ void owl_function_info()
 
   owl_fmtext_append_bold(&fm, "General Information:\n");
   owl_fmtext_append_normal(&fm, "  Msg Id    : ");
-  sprintf(buff, "%i", owl_message_get_id(m));
+  buff=owl_sprintf("%i", owl_message_get_id(m));
   owl_fmtext_append_normal(&fm, buff);
+  owl_free(buff);
   owl_fmtext_append_normal(&fm, "\n");
 
   owl_fmtext_append_normal(&fm, "  Type      : ");
@@ -1592,26 +1498,31 @@ void owl_function_info()
 
       if (!owl_message_is_pseudo(m)) {
 	owl_fmtext_append_normal(&fm, "\n");
-	sprintf(buff, "  Port      : %i\n", ntohs(n->z_port));
+	buff=owl_sprintf("  Port      : %i\n", ntohs(n->z_port));
 	owl_fmtext_append_normal(&fm, buff);
+	owl_free(buff);
 
 	owl_fmtext_append_normal(&fm,    "  Auth      : ");
 	owl_fmtext_append_normal(&fm, owl_zephyr_get_authstr(n));
 	owl_fmtext_append_normal(&fm, "\n");
 	
-	/* fix this */
-	sprintf(buff, "  Checkd Ath: %i\n", n->z_checked_auth);
-	sprintf(buff, "%s  Multi notc: %s\n", buff, n->z_multinotice);
-	sprintf(buff, "%s  Num other : %i\n", buff, n->z_num_other_fields);
-	sprintf(buff, "%s  Msg Len   : %i\n", buff, n->z_message_len);
+	buff=owl_sprintf("  Checkd Ath: %i\n  Multi notc: %s\n  Num other : %i\n  Msg Len   : %i\n",
+			 n->z_checked_auth,
+			 n->z_multinotice,
+			 n->z_num_other_fields,
+			 n->z_message_len);
 	owl_fmtext_append_normal(&fm, buff);
+	owl_free(buff);
 	
-	sprintf(buff, "  Fields    : %i\n", owl_zephyr_get_num_fields(n));
+	buff=owl_sprintf("  Fields    : %i\n", owl_zephyr_get_num_fields(n));
 	owl_fmtext_append_normal(&fm, buff);
+	owl_free(buff);
 	
 	fields=owl_zephyr_get_num_fields(n);
 	for (i=0; i<fields; i++) {
-	  sprintf(buff, "  Field %i   : ", i+1);
+	  buff=owl_sprintf("  Field %i   : ", i+1);
+	  owl_fmtext_append_normal(&fm, buff);
+	  owl_free(buff);
 	  
 	  ptr=owl_zephyr_get_field(n, i+1);
 	  len=strlen(ptr);
@@ -1630,9 +1541,8 @@ void owl_function_info()
 	    if (tmpbuff[j]=='\r') tmpbuff[j]='!';
 	  }
 	  
-	  strcat(buff, tmpbuff);
-	  strcat(buff, "\n");
-	  owl_fmtext_append_normal(&fm, buff);
+	  owl_fmtext_append_normal(&fm, tmpbuff);
+	  owl_fmtext_append_normal(&fm, "\n");
 	}
 	owl_fmtext_append_normal(&fm, "  Default Fm:");
 	owl_fmtext_append_normal(&fm, n->z_default_format);
@@ -1798,34 +1708,31 @@ void owl_function_getsubs()
   owl_free(buff);
 }
 
-#define PABUFLEN 5000
 void owl_function_printallvars()
 {
-  char buff[PABUFLEN], *pos, *name;
+  char *name;
+  char var[LINE];
   owl_list varnames;
-  int i, numvarnames, rem;
+  int i, numvarnames;
+  GString *str   = g_string_new("");
 
-  pos = buff;
-  pos += sprintf(pos, "%-20s = %s\n", "VARIABLE", "VALUE");
-  pos += sprintf(pos, "%-20s   %s\n",  "--------", "-----");
+  g_string_append_printf(str, "%-20s = %s\n", "VARIABLE", "VALUE");
+  g_string_append_printf(str, "%-20s   %s\n",  "--------", "-----");
   owl_variable_dict_get_names(owl_global_get_vardict(&g), &varnames);
-  rem = (buff+PABUFLEN)-pos-1;
   numvarnames = owl_list_get_size(&varnames);
   for (i=0; i<numvarnames; i++) {
     name = owl_list_get_element(&varnames, i);
     if (name && name[0]!='_') {
-      rem = (buff+PABUFLEN)-pos-1;    
-      pos += snprintf(pos, rem, "\n%-20s = ", name);
-      rem = (buff+PABUFLEN)-pos-1;    
-      owl_variable_get_tostring(owl_global_get_vardict(&g), name, pos, rem);
-      pos = buff+strlen(buff);
+      g_string_append_printf(str, "\n%-20s = ", name);
+      owl_variable_get_tostring(owl_global_get_vardict(&g), name, var, LINE);
+      g_string_append(str, var);
     }
   }
-  rem = (buff+PABUFLEN)-pos-1;    
-  snprintf(pos, rem, "\n");
+  g_string_append(str, "\n");
   owl_variable_dict_namelist_free(&varnames);
-  
-  owl_function_popless_text(buff);
+
+  owl_function_popless_text(str->str);
+  g_string_free(str, TRUE);
 }
 
 void owl_function_show_variables()
@@ -1917,7 +1824,8 @@ void owl_function_delete_automsgs()
 
 void owl_function_status()
 {
-  char buff[5000];
+  char cwdbuff[MAXPATHLEN+1];
+  char *buff;
   time_t start;
   int up, days, hours, minutes;
   owl_fmtext fm;
@@ -1937,12 +1845,16 @@ void owl_function_status()
   owl_fmtext_append_normal(&fm, "\n");
 
   owl_fmtext_append_normal(&fm, "  Current Directory: ");
-  (void) getcwd(buff, MAXPATHLEN);
-  owl_fmtext_append_normal(&fm, buff);
+  if(getcwd(cwdbuff, MAXPATHLEN) == NULL) {
+    owl_fmtext_append_normal(&fm, "<Error in getcwd>");
+  } else {
+    owl_fmtext_append_normal(&fm, cwdbuff);
+  }
   owl_fmtext_append_normal(&fm, "\n");
 
-  sprintf(buff, "  Startup Time: %s", ctime(&start));
+  buff=owl_sprintf("  Startup Time: %s", ctime(&start));
   owl_fmtext_append_normal(&fm, buff);
+  owl_free(buff);
 
   up=owl_global_get_runtime(&g);
   days=up/86400;
@@ -1951,8 +1863,10 @@ void owl_function_status()
   up-=hours*3600;
   minutes=up/60;
   up-=minutes*60;
-  sprintf(buff, "  Run Time: %i days %2.2i:%2.2i:%2.2i\n", days, hours, minutes, up);
+
+  buff=owl_sprintf("  Run Time: %i days %2.2i:%2.2i:%2.2i\n", days, hours, minutes, up);
   owl_fmtext_append_normal(&fm, buff);
+  owl_free(buff);
 
   owl_fmtext_append_normal(&fm, "\nProtocol Options:\n");
   owl_fmtext_append_normal(&fm, "  Zephyr included    : ");
@@ -1976,11 +1890,6 @@ void owl_function_status()
 
   owl_fmtext_append_normal(&fm, "\nMemory Usage:\n");
   owl_fmtext_append_normal(&fm, "  Not currently available.\n");
-  /*
-  sprintf(buff, "%sMemory Malloced: %i\n", buff, owl_global_get_malloced(&g));
-  sprintf(buff, "%sMemory Freed: %i\n", buff, owl_global_get_freed(&g));
-  sprintf(buff, "%sMemory In Use: %i\n", buff, owl_global_get_meminuse(&g));
-  */
 
   owl_fmtext_append_normal(&fm, "\nAIM Status:\n");
   owl_fmtext_append_normal(&fm, "  Logged in: ");
@@ -2005,20 +1914,23 @@ void owl_function_status()
 void owl_function_show_term()
 {
   owl_fmtext fm;
-  char buff[LINE];
+  char *buff;
 
   owl_fmtext_init_null(&fm);
-  sprintf(buff, "Terminal Lines: %i\nTerminal Columns: %i\n",
-	  owl_global_get_lines(&g),
-	  owl_global_get_cols(&g));
+  buff=owl_sprintf("Terminal Lines: %i\nTerminal Columns: %i\n",
+		   owl_global_get_lines(&g),
+		   owl_global_get_cols(&g));
   owl_fmtext_append_normal(&fm, buff);
+  owl_free(buff);
 
   if (owl_global_get_hascolors(&g)) {
     owl_fmtext_append_normal(&fm, "Color: Yes\n");
-    sprintf(buff, "Number of color pairs: %i\n", owl_global_get_colorpairs(&g));
+    buff=owl_sprintf("Number of color pairs: %i\n", owl_global_get_colorpairs(&g));
     owl_fmtext_append_normal(&fm, buff);
-    sprintf(buff, "Can change colors: %s\n", can_change_color() ? "yes" : "no");
+    owl_free(buff);
+    buff=owl_sprintf("Can change colors: %s\n", can_change_color() ? "yes" : "no");
     owl_fmtext_append_normal(&fm, buff);
+    owl_free(buff);
   } else {
     owl_fmtext_append_normal(&fm, "Color: No\n");
   }
@@ -2034,7 +1946,7 @@ void owl_function_show_term()
  */
 void owl_function_reply(int type, int enter)
 {
-  char *buff=NULL, *oldbuff;
+  char *buff=NULL, *tmpbuff;
   owl_message *m;
   owl_filter *f;
   
@@ -2118,15 +2030,17 @@ void owl_function_reply(int type, int enter)
       if (!strcasecmp(owl_message_get_opcode(m), "CRYPT")) {
 	buff=owl_strdup("zcrypt");
       } else {
-	buff = owl_strdup("zwrite");
+	buff=owl_strdup("zwrite");
       }
       if (strcasecmp(class, "message")) {
-	buff = owl_sprintf("%s -c %s%s%s", oldbuff=buff, owl_getquoting(class), class, owl_getquoting(class));
-	owl_free(oldbuff);
+	tmpbuff=owl_sprintf("%s -c %s%s%s", buff, owl_getquoting(class), class, owl_getquoting(class));
+	owl_free(buff);
+	buff=tmpbuff;
       }
       if (strcasecmp(inst, "personal")) {
-	buff = owl_sprintf("%s -i %s%s%s", oldbuff=buff, owl_getquoting(inst), inst, owl_getquoting(inst));
-	owl_free(oldbuff);
+	tmpbuff=owl_sprintf("%s -i %s%s%s", buff, owl_getquoting(inst), inst, owl_getquoting(inst));
+	owl_free(buff);
+	buff=tmpbuff;
       }
       if (*to != '\0') {
 	char *tmp, *oldtmp, *tmp2;
@@ -2134,16 +2048,18 @@ void owl_function_reply(int type, int enter)
 	if (cc) {
 	  tmp = owl_util_uniq(oldtmp=tmp, cc, "-");
 	  owl_free(oldtmp);
-	  buff = owl_sprintf("%s -C %s", oldbuff=buff, tmp);
-	  owl_free(oldbuff);
+	  tmpbuff=owl_sprintf("%s -C %s", buff, tmp);
+	  owl_free(buff);
+	  buff=tmpbuff;
 	} else {
 	  if (owl_global_is_smartstrip(&g)) {
 	    tmp2=tmp;
 	    tmp=owl_zephyr_smartstripped_user(tmp2);
 	    owl_free(tmp2);
 	  }
-	  buff = owl_sprintf("%s %s", oldbuff=buff, tmp);
-	  owl_free(oldbuff);
+	  tmpbuff=owl_sprintf("%s %s", buff, tmp);
+	  owl_free(buff);
+	  buff=tmpbuff;
 	}
 	owl_free(tmp);
       }
@@ -2174,16 +2090,17 @@ void owl_function_reply(int type, int enter)
 void owl_function_zlocate(int argc, char **argv, int auth)
 {
   owl_fmtext fm;
-  char *ptr, buff[LINE];
+  char *ptr, *buff;
   int i;
 
   owl_fmtext_init_null(&fm);
 
   for (i=0; i<argc; i++) {
     ptr=long_zuser(argv[i]);
-    owl_zephyr_zlocate(ptr, buff, auth);
+    buff=owl_zephyr_zlocate(ptr, auth);
     owl_fmtext_append_normal(&fm, buff);
     owl_free(ptr);
+    owl_free(buff);
   }
 
   owl_function_popless_fmtext(&fm);
@@ -2523,15 +2440,18 @@ void owl_function_show_filters()
 void owl_function_show_filter(char *name)
 {
   owl_filter *f;
-  char buff[5000];
+  char *buff, *tmp;
 
   f=owl_global_get_filter(&g, name);
   if (!f) {
     owl_function_error("There is no filter named %s", name);
     return;
   }
-  owl_filter_print(f, buff);
+  tmp = owl_filter_print(f);
+  buff = owl_sprintf("%s: %s", owl_filter_get_name(f), tmp);
   owl_function_popless_text(buff);
+  owl_free(buff);
+  owl_free(tmp);
 }
 
 void owl_function_show_zpunts()
@@ -2539,6 +2459,7 @@ void owl_function_show_zpunts()
   owl_filter *f;
   owl_list *fl;
   char buff[5000];
+  char *tmp;
   owl_fmtext fm;
   int i, j;
 
@@ -2550,8 +2471,12 @@ void owl_function_show_zpunts()
 
   for (i=0; i<j; i++) {
     f=owl_list_get_element(fl, i);
-    owl_filter_print(f, buff);
+    snprintf(buff, sizeof(buff), "[% 2d] ", i+1);
     owl_fmtext_append_normal(&fm, buff);
+    tmp = owl_filter_print(f);
+    owl_fmtext_append_normal(&fm, tmp);
+    owl_free(tmp);
+    owl_fmtext_append_normal(&fm, "\n");
   }
   owl_function_popless_fmtext(&fm);
   owl_fmtext_free(&fm);
@@ -2561,29 +2486,39 @@ void owl_function_show_zpunts()
  * instance is NULL then catch all messgaes in the class.  Returns the
  * name of the filter, which the caller must free.
  */
-char *owl_function_classinstfilt(char *class, char *instance) 
+char *owl_function_classinstfilt(char *c, char *i) 
 {
   owl_list *fl;
   owl_filter *f;
   char *argbuff, *filtname;
   char *tmpclass, *tmpinstance = NULL;
-  int len;
+  char *class, *instance = NULL;
+
+  class = owl_util_baseclass(c);
+  if(i) {
+    instance = owl_util_baseclass(i);
+  }
 
   fl=owl_global_get_filterlist(&g);
 
   /* name for the filter */
-  len=strlen(class)+30;
-  if (instance) len+=strlen(instance);
-  filtname=owl_malloc(len);
   if (!instance) {
-    sprintf(filtname, "class-%s", class);
+    filtname = owl_sprintf("class-%s", class);
   } else {
-    sprintf(filtname, "class-%s-instance-%s", class, instance);
+    filtname = owl_sprintf("class-%s-instance-%s", class, instance);
   }
   /* downcase it */
-  downstr(filtname);
-  /* turn spaces into dots */
+  {
+    char *temp = g_utf8_strdown(filtname, -1);
+    if (temp) {
+      owl_free(filtname);
+      filtname = temp;
+    }
+  }
+  /* turn spaces, single quotes, and double quotes into dots */
   owl_text_tr(filtname, ' ', '.');
+  owl_text_tr(filtname, '\'', '.');
+  owl_text_tr(filtname, '"', '.');
   
   /* if it already exists then go with it.  This lets users override */
   if (owl_global_get_filter(&g, filtname)) {
@@ -2591,16 +2526,22 @@ char *owl_function_classinstfilt(char *class, char *instance)
   }
 
   /* create the new filter */
-  argbuff=owl_malloc(len+20);
   tmpclass=owl_text_quote(class, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
   owl_text_tr(tmpclass, ' ', '.');
+  owl_text_tr(tmpclass, '\'', '.');
+  owl_text_tr(tmpclass, '"', '.');
   if (instance) {
     tmpinstance=owl_text_quote(instance, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
     owl_text_tr(tmpinstance, ' ', '.');
+    owl_text_tr(tmpinstance, '\'', '.');
+    owl_text_tr(tmpinstance, '"', '.');
   }
-  sprintf(argbuff, "( class ^%s$ )", tmpclass);
+
+  argbuff = owl_sprintf("class ^(un)*%s(\\.d)*$", tmpclass);
   if (tmpinstance) {
-    sprintf(argbuff, "%s and ( instance ^%s$ )", argbuff, tmpinstance);
+    char *tmp = argbuff;
+    argbuff = owl_sprintf("%s and ( instance ^(un)*%s(\\.d)*$ )", tmp, tmpinstance);
+    owl_free(tmp);
   }
   owl_free(tmpclass);
   if (tmpinstance) owl_free(tmpinstance);
@@ -2612,6 +2553,10 @@ char *owl_function_classinstfilt(char *class, char *instance)
   owl_global_add_filter(&g, f);
 
   owl_free(argbuff);
+  owl_free(class);
+  if (instance) {
+    owl_free(instance);
+  }
   return(filtname);
 }
 
@@ -2625,15 +2570,14 @@ char *owl_function_classinstfilt(char *class, char *instance)
 char *owl_function_zuserfilt(char *user)
 {
   owl_filter *f;
-  char *argbuff, *longuser, *shortuser, *filtname;
+  char *argbuff, *longuser, *esclonguser, *shortuser, *filtname;
 
   /* stick the local realm on if it's not there */
   longuser=long_zuser(user);
   shortuser=short_zuser(user);
 
   /* name for the filter */
-  filtname=owl_malloc(strlen(shortuser)+20);
-  sprintf(filtname, "user-%s", shortuser);
+  filtname=owl_sprintf("user-%s", shortuser);
 
   /* if it already exists then go with it.  This lets users override */
   if (owl_global_get_filter(&g, filtname)) {
@@ -2643,10 +2587,12 @@ char *owl_function_zuserfilt(char *user)
   /* create the new-internal filter */
   f=owl_malloc(sizeof(owl_filter));
 
-  argbuff=owl_malloc(strlen(longuser)+1000);
-  sprintf(argbuff, "( type ^zephyr$ and ( class ^message$ and instance ^personal$ and ");
-  sprintf(argbuff, "%s ( ( direction ^in$ and sender ^%s$ ) or ( direction ^out$ and recipient ^%s$ ) ) )", argbuff, longuser, longuser);
-  sprintf(argbuff, "%s or ( ( class ^login$ ) and ( sender ^%s$ ) ) )", argbuff, longuser);
+  esclonguser = owl_text_quote(longuser, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
+
+  argbuff=owl_sprintf("( type ^zephyr$ and filter personal and "
+      "( ( direction ^in$ and sender ^%1$s$ ) or ( direction ^out$ and "
+      "recipient ^%1$s$ ) ) ) or ( ( class ^login$ ) and ( sender ^%1$s$ ) )",
+      esclonguser);
 
   owl_filter_init_fromstring(f, filtname, argbuff);
 
@@ -2656,6 +2602,7 @@ char *owl_function_zuserfilt(char *user)
   /* free stuff */
   owl_free(argbuff);
   owl_free(longuser);
+  owl_free(esclonguser);
   owl_free(shortuser);
 
   return(filtname);
@@ -2671,10 +2618,10 @@ char *owl_function_aimuserfilt(char *user)
 {
   owl_filter *f;
   char *argbuff, *filtname;
+  char *escuser;
 
   /* name for the filter */
-  filtname=owl_malloc(strlen(user)+40);
-  sprintf(filtname, "aimuser-%s", user);
+  filtname=owl_sprintf("aimuser-%s", user);
 
   /* if it already exists then go with it.  This lets users override */
   if (owl_global_get_filter(&g, filtname)) {
@@ -2684,10 +2631,12 @@ char *owl_function_aimuserfilt(char *user)
   /* create the new-internal filter */
   f=owl_malloc(sizeof(owl_filter));
 
-  argbuff=owl_malloc(1000);
-  sprintf(argbuff,
-	  "( type ^aim$ and ( ( sender ^%s$ and recipient ^%s$ ) or ( sender ^%s$ and recipient ^%s$ ) ) )",
-	  user, owl_global_get_aim_screenname(&g), owl_global_get_aim_screenname(&g), user);
+  escuser = owl_text_quote(user, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
+
+  argbuff = owl_sprintf(
+      "( type ^aim$ and ( ( sender ^%1$s$ and recipient ^%2$s$ ) or "
+      "( sender ^%2$s$ and recipient ^%1$s$ ) ) )",
+      escuser, owl_global_get_aim_screenname(&g));
 
   owl_filter_init_fromstring(f, filtname, argbuff);
 
@@ -2696,6 +2645,7 @@ char *owl_function_aimuserfilt(char *user)
 
   /* free stuff */
   owl_free(argbuff);
+  owl_free(escuser);
 
   return(filtname);
 }
@@ -2959,45 +2909,61 @@ void owl_function_show_colors()
  *   if direction==1 then unpunt
  */
 void owl_function_zpunt(char *class, char *inst, char *recip, int direction)
+{
+  char *puntexpr, *classexpr, *instexpr, *recipexpr;
+  char *quoted;
+
+  if (!strcmp(class, "*")) {
+    classexpr = owl_sprintf("class .*");
+  } else {
+    quoted=owl_text_quote(class, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
+    owl_text_tr(quoted, ' ', '.');
+    owl_text_tr(quoted, '\'', '.');
+    owl_text_tr(quoted, '"', '.');
+    classexpr = owl_sprintf("class ^(un)*%s(\\.d)*$", quoted);
+    owl_free(quoted);
+  }
+  if (!strcmp(inst, "*")) {
+    instexpr = owl_sprintf(" and instance .*");
+  } else {
+    quoted=owl_text_quote(inst, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
+    owl_text_tr(quoted, ' ', '.');
+    owl_text_tr(quoted, '\'', '.');
+    owl_text_tr(quoted, '"', '.');
+    instexpr = owl_sprintf(" and instance ^(un)*%s(\\.d)*$", quoted);
+    owl_free(quoted);
+  }
+  if (!strcmp(recip, "*")) {
+    recipexpr = owl_sprintf("");
+  } else {
+    quoted=owl_text_quote(recip, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
+    owl_text_tr(quoted, ' ', '.');
+    owl_text_tr(quoted, '\'', '.');
+    owl_text_tr(quoted, '"', '.');
+    recipexpr = owl_sprintf(" and recipient ^%s$", quoted);
+    owl_free(quoted);
+  }
+
+  puntexpr = owl_sprintf("%s %s %s", classexpr, instexpr, recipexpr);
+  owl_function_punt(puntexpr, direction);
+  owl_free(puntexpr);
+  owl_free(classexpr);
+  owl_free(instexpr);
+  owl_free(recipexpr);
+}
+
+void owl_function_punt(char *filter, int direction)
 {
   owl_filter *f;
   owl_list *fl;
-  char *buff;
-  char *quoted;
   int ret, i, j;
-
   fl=owl_global_get_puntlist(&g);
 
   /* first, create the filter */
   f=malloc(sizeof(owl_filter));
-  buff=malloc(strlen(class)+strlen(inst)+strlen(recip)+100);
-  strcpy(buff, "class");
-  if (!strcmp(class, "*")) {
-    strcat(buff, " .*");
-  } else {
-    quoted=owl_text_quote(class, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
-    owl_text_tr(quoted, ' ', '.');
-    sprintf(buff, "%s ^%s$", buff, quoted);
-    owl_free(quoted);
-  }
-  if (!strcmp(inst, "*")) {
-    strcat(buff, " and instance .*");
-  } else {
-    quoted=owl_text_quote(inst, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
-    owl_text_tr(quoted, ' ', '.');
-    sprintf(buff, "%s and instance ^%s$", buff, quoted);
-    owl_free(quoted);
-  }
-  if (strcmp(recip, "*")) {
-    quoted=owl_text_quote(recip, OWL_REGEX_QUOTECHARS, OWL_REGEX_QUOTEWITH);
-    owl_text_tr(quoted, ' ', '.');
-    sprintf(buff, "%s and recipient ^%s$", buff, quoted);
-    owl_free(quoted);
-  }
-  
-  owl_function_debugmsg("About to filter %s", buff);
-  ret=owl_filter_init_fromstring(f, "punt-filter", buff);
-  owl_free(buff);
+
+  owl_function_debugmsg("About to filter %s", filter);
+  ret=owl_filter_init_fromstring(f, "punt-filter", filter);
   if (ret) {
     owl_function_error("Error creating filter for zpunt");
     owl_filter_free(f);
@@ -3008,6 +2974,7 @@ void owl_function_zpunt(char *class, char *inst, char *recip, int direction)
   j=owl_list_get_size(fl);
   for (i=0; i<j; i++) {
     if (owl_filter_equiv(f, owl_list_get_element(fl, i))) {
+      owl_function_debugmsg("found an equivalent punt filter");
       /* if we're punting, then just silently bow out on this duplicate */
       if (direction==0) {
 	owl_filter_free(f);
@@ -3018,11 +2985,13 @@ void owl_function_zpunt(char *class, char *inst, char *recip, int direction)
       if (direction==1) {
 	owl_filter_free(owl_list_get_element(fl, i));
 	owl_list_remove_element(fl, i);
+        owl_filter_free(f);
 	return;
       }
     }
   }
 
+  owl_function_debugmsg("punting");
   /* If we're punting, add the filter to the global punt list */
   if (direction==0) {
     owl_list_append_element(fl, f);
@@ -3249,15 +3218,15 @@ void owl_function_buddylist(int aim, int zephyr, char *filename)
 	ret=ZGetLocations(location, &numlocs);
 	if (ret==0) {
 	  for (x=0; x<numlocs; x++) {
-	    line=malloc(strlen(location[x].host)+strlen(location[x].time)+strlen(location[x].tty)+100);
 	    tmp=short_zuser(user);
-	    sprintf(line, "  %-10.10s %-24.24s %-12.12s  %20.20s\n",
-		    tmp,
-		    location[x].host,
-		    location[x].tty,
-		    location[x].time);
+	    line=owl_sprintf("  %-10.10s %-24.24s %-12.12s  %20.20s\n",
+			     tmp,
+			     location[x].host,
+			     location[x].tty,
+			     location[x].time);
 	    owl_fmtext_append_normal(&fm, line);
 	    owl_free(tmp);
+	    owl_free(line);
 	  }
 	  if (numlocs>=200) {
 	    owl_fmtext_append_normal(&fm, "  Too many locations found for this user, truncating.\n");
@@ -3416,21 +3385,27 @@ void owl_function_delstartup(char *buff)
  */
 void owl_function_source(char *filename)
 {
+  char *path;
   FILE *file;
   char buff[LINE];
+  int fail_silent = 0;
 
   if (!filename) {
-    filename=owl_sprintf("%s/%s", owl_global_get_homedir(&g), OWL_STARTUP_FILE);
-    file=fopen(filename, "r");
-    owl_free(filename);
+    fail_silent = 1;
+    path = owl_sprintf("%s/%s", owl_global_get_homedir(&g), OWL_STARTUP_FILE);
   } else {
-    file=fopen(filename, "r");
+    path = owl_util_makepath(filename);
   }
+  file=fopen(path, "r");
+  owl_free(path);
   if (!file) {
-    /* just fail silently if it doesn't exist */
+    if (!fail_silent) {
+      owl_function_error("Error opening file: %s", filename);
+    }
     return;
   }
   while (fgets(buff, LINE, file)!=NULL) {
+    if (buff[0] == '#') continue;
     buff[strlen(buff)-1]='\0';
     owl_function_command(buff);
   }
@@ -3474,7 +3449,8 @@ void owl_function_toggleoneline()
 void owl_function_error(char *fmt, ...)
 {
   va_list ap;
-  char buff[2048], buff2[2048];
+  char *buff, *buff2;
+  char *nl;
   char *date;
   time_t now;
 
@@ -3484,18 +3460,21 @@ void owl_function_error(char *fmt, ...)
 
   va_start(ap, fmt);
 
-  vsnprintf(buff, 2048, fmt, ap);
-  sprintf(buff2, "%s %s", date, buff);
+  buff = g_strdup_vprintf(fmt, ap);
+  buff2 = owl_sprintf("%s %s", date, buff);
   owl_function_debugmsg("ERROR: %s", buff);
-  if (owl_global_get_curs_msgwin(&g)) {
-    werase(owl_global_get_curs_msgwin(&g));
-    waddstr(owl_global_get_curs_msgwin(&g), buff);  
-    wnoutrefresh(owl_global_get_curs_msgwin(&g));
-    owl_global_set_needrefresh(&g);
+  nl = strchr(buff, '\n');
+  if(nl && *(nl + 1)) {
+    /* Multiline error */
+    owl_function_adminmsg("ERROR", buff);
+  } else {
+    owl_function_makemsg("[Error] %s", buff);
   }
   owl_errqueue_append_err(owl_global_get_errqueue(&g), buff2);
   va_end(ap);
   owl_free(date);
+  owl_free(buff);
+  owl_free(buff2);
 }
 
 void owl_function_showerrs()
