@@ -1,3 +1,27 @@
+/* Copyright (c) 2002,2003,2004,2009 James M. Kretchmar
+ *
+ * This file is part of Owl.
+ *
+ * Owl is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Owl is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Owl.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * ---------------------------------------------------------------
+ * 
+ * As of Owl version 2.1.12 there are patches contributed by
+ * developers of the branched BarnOwl project, Copyright (c)
+ * 2006-2009 The BarnOwl Developers. All rights reserved.
+ */
+
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -8,7 +32,7 @@
 #include <time.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: global.c,v 1.35 2005/01/30 17:26:44 kretch Exp $";
+static const char fileIdent[] = "$Id: global.c,v 1.42 2009/04/07 15:41:22 kretch Exp $";
 
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 256
@@ -49,7 +73,6 @@ void owl_global_init(owl_global *g) {
   owl_keyhandler_init(&g->kh);
   owl_keys_setup_keymaps(&g->kh);
 
-  owl_list_create(&(g->muxevents));
   owl_list_create(&(g->filterlist));
   owl_list_create(&(g->puntlist));
   owl_list_create(&(g->messagequeue));
@@ -109,6 +132,8 @@ void owl_global_init(owl_global *g) {
 
   owl_zbuddylist_create(&(g->zbuddies));
   owl_timer_create_countdown(&(g->zephyr_buddycheck_timer), 60*3);
+
+  owl_list_create(&(g->dispatchlist));
 }
 
 void _owl_global_setup_windows(owl_global *g) {
@@ -436,13 +461,13 @@ time_t owl_global_get_runtime(owl_global *g) {
   return(time(NULL)-g->starttime);
 }
 
-void owl_global_get_runtime_string(owl_global *g, char *buff) {
+char *owl_global_get_runtime_string(owl_global *g) {
   time_t diff;
 
   diff=time(NULL)-owl_global_get_starttime(g);
 
   /* print something nicer later */   
-  sprintf(buff, "%i seconds", (int) diff);
+  return(owl_sprintf("%i seconds", (int) diff));
 }
 
 char *owl_global_get_hostname(owl_global *g) {
@@ -497,11 +522,11 @@ void owl_global_set_startupargs(owl_global *g, int argc, char **argv) {
   for (i=0; i<argc; i++) {
     len+=strlen(argv[i])+5;
   }
-  g->startupargs=malloc(len+5);
+  g->startupargs=owl_malloc(len+5);
 
   strcpy(g->startupargs, "");
   for (i=0; i<argc; i++) {
-    sprintf(g->startupargs, "%s%s ", g->startupargs, argv[i]);
+    sprintf(g->startupargs + strlen(g->startupargs), "%s ", argv[i]);
   }
   g->startupargs[strlen(g->startupargs)-1]='\0';
 }
@@ -521,12 +546,6 @@ owl_history *owl_global_get_cmd_history(owl_global *g) {
   return(&(g->cmdhist));
 }
 
-/* muxevents */
-
-owl_muxevents *owl_global_get_muxevents(owl_global *g) {
-  return(&(g->muxevents));
-}
-
 /* filterlist */
 
 owl_list *owl_global_get_filterlist(owl_global *g) {
@@ -888,3 +907,8 @@ struct termios *owl_global_get_startup_tio(owl_global *g)
 {
   return(&(g->startup_tio));
 }
+
+owl_list *owl_global_get_dispatchlist(owl_global *g)
+{
+  return &(g->dispatchlist);
+}
