@@ -1,6 +1,30 @@
+/* Copyright (c) 2002,2003,2004,2009 James M. Kretchmar
+ *
+ * This file is part of Owl.
+ *
+ * Owl is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Owl is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Owl.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * ---------------------------------------------------------------
+ * 
+ * As of Owl version 2.1.12 there are patches contributed by
+ * developers of the branched BarnOwl project, Copyright (c)
+ * 2006-2009 The BarnOwl Developers. All rights reserved.
+ */
+
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: keys.c,v 1.24 2003/12/25 05:05:02 kretch Exp $";
+static const char fileIdent[] = "$Id: keys.c,v 1.26 2009/03/29 12:38:21 kretch Exp $";
 
 #define BIND_CMD(kpress, command, desc) \
          owl_keymap_create_binding(km, kpress, command, NULL, desc);
@@ -274,8 +298,6 @@ void owl_keys_setup_keymaps(owl_keyhandler *kh) {
   BIND_CMD("C-r", "reply -e",         "reply to the current message, but allow editing of recipient");
   BIND_CMD("M-r", "reply -e",         "reply to the current message, but allow editing of recipient");
   BIND_CMD("M-R", "reply -e sender",  "reply to sender of the current message, but allow editing of recipient");
-		  
-  BIND_CMD("w",   "openurl",          "open a URL using a webbrowser");
 
   BIND_CMD("C-c",  "",                "no effect in this mode");
   BIND_CMD("C-g",  "",                "no effect in this mode");
