@@ -1,3 +1,27 @@
+/* Copyright (c) 2002,2003,2004,2009 James M. Kretchmar
+ *
+ * This file is part of Owl.
+ *
+ * Owl is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Owl is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Owl.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * ---------------------------------------------------------------
+ * 
+ * As of Owl version 2.1.12 there are patches contributed by
+ * developers of the branched BarnOwl project, Copyright (c)
+ * 2006-2009 The BarnOwl Developers. All rights reserved.
+ */
+
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
@@ -10,7 +34,7 @@
 #include <time.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: message.c,v 1.50 2004/12/30 20:38:59 kretch Exp $";
+static const char fileIdent[] = "$Id: message.c,v 1.53 2009/04/05 23:36:54 kretch Exp $";
 
 void owl_message_init(owl_message *m)
 {
@@ -19,7 +43,7 @@ void owl_message_init(owl_message *m)
   owl_message_set_direction_none(m);
   m->delete=0;
   m->hostname=owl_strdup("");
-  m->zwriteline=strdup("");
+  m->zwriteline=NULL;
   m->invalid_format=1;
 
   owl_list_create(&(m->attributes));
@@ -436,11 +460,14 @@ void owl_message_unmark_delete(owl_message *m)
 
 char *owl_message_get_zwriteline(owl_message *m)
 {
+  if(!m->zwriteline)
+    return "";
   return(m->zwriteline);
 }
 
 void owl_message_set_zwriteline(owl_message *m, char *line)
 {
+  if(m->zwriteline) owl_free(m->zwriteline);
   m->zwriteline=strdup(line);
 }
 
@@ -562,26 +589,28 @@ int owl_message_is_ping(owl_message *m)
 
 int owl_message_is_burningears(owl_message *m)
 {
-  /* we should add a global to cache the short zsender */
-  char sender[LINE], *ptr;
+  char *name;
+  int ret;
 
   /* if the message is from us or to us, it doesn't count */
   if (owl_message_is_from_me(m) || owl_message_is_private(m)) return(0);
 
   if (owl_message_is_type_zephyr(m)) {
-    strcpy(sender, owl_zephyr_get_sender());
-    ptr=strchr(sender, '@');
-    if (ptr) *ptr='\0';
+    name=short_zuser(owl_zephyr_get_sender());
   } else if (owl_message_is_type_aim(m)) {
-    strcpy(sender, owl_global_get_aim_screenname(&g));
+    name=owl_strdup(owl_global_get_aim_screenname(&g));
   } else {
     return(0);
   }
 
-  if (stristr(owl_message_get_body(m), sender)) {
-    return(1);
+  if (stristr(owl_message_get_body(m), name)) {
+    ret=1;
+  } else {
+    ret=0;
   }
-  return(0);
+
+  owl_free(name);
+  return(ret);
 }
 
 /* caller must free return value. */
@@ -723,6 +752,7 @@ void owl_message_create_from_znotice(owl_message *m, ZNotice_t *n)
 {
   struct hostent *hent;
   char *ptr, *tmp, *tmp2;
+  int len;
 
   owl_message_init(m);
   
@@ -751,7 +781,7 @@ void owl_message_create_from_znotice(owl_message *m, ZNotice_t *n)
   } else {
     owl_message_set_opcode(m, "");
   }
-  owl_message_set_zsig(m, n->z_message);
+  owl_message_set_zsig(m, owl_zephyr_get_zsig(n, &len));
 
   if ((ptr=strchr(n->z_recipient, '@'))!=NULL) {
     owl_message_set_realm(m, ptr+1);
