@@ -27,6 +27,7 @@ extern int logout(aim_session_t *sess);
 extern int faimtest_parse_genericerr(aim_session_t *sess, aim_frame_t *fr, ...);
 extern void chatnav_redirect(aim_session_t *sess, struct aim_redirect_data *redir);
 extern void chat_redirect(aim_session_t *sess, struct aim_redirect_data *redir);
+extern void owl_process_aim();
 #if 0
 #endif
 
@@ -150,7 +151,6 @@ extern char *owl_command_show(int argc, char **argv, char *buff);
 extern char *owl_command_viewclass(int argc, char **argv, char *buff);
 extern char *owl_command_viewuser(int argc, char **argv, char *buff);
 extern void owl_command_pop_message(void);
-extern void owl_command_openurl(void);
 extern char *owl_command_delete(int argc, char **argv, char *buff);
 extern char *owl_command_undelete(int argc, char **argv, char *buff);
 extern void owl_command_beep();
@@ -165,6 +165,7 @@ extern char *owl_command_getvar(int argc, char **argv, char *buff);
 extern char *owl_command_search(int argc, char **argv, char *buff);
 extern char *owl_command_aimlogin(int argc, char **argv, char *buff);
 extern char *owl_command_aimlogout(int argc, char **argv, char *buff);
+extern char *owl_command_getstyle(int argc, char **argv, char *buff);
 extern void owl_command_edit_cancel(owl_editwin *e);
 extern void owl_command_edit_history_prev(owl_editwin *e);
 extern void owl_command_edit_history_next(owl_editwin *e);
@@ -274,7 +275,7 @@ extern void owl_filter_set_cachedmsgid(owl_filter *f, int cachedmsgid);
 extern int owl_filter_get_cachedmsgid(owl_filter *f);
 extern int owl_filter_message_match(owl_filter *f, owl_message *m);
 extern int _owl_filter_message_match_recurse(owl_filter *f, owl_message *m, owl_list *fes, int start, int end);
-extern void owl_filter_print(owl_filter *f, char *out);
+extern char *owl_filter_print(owl_filter *f);
 extern int owl_filter_equiv(owl_filter *a, owl_filter *b);
 extern void _owl_filter_get_subfilter_names(owl_filter *f, owl_list *list);
 extern int owl_filter_is_toodeep(owl_filter *f);
@@ -398,7 +399,6 @@ extern void owl_function_aaway_toggle();
 extern void owl_function_aaway_on();
 extern void owl_function_aaway_off();
 extern void owl_function_quit();
-extern void owl_function_openurl();
 extern void owl_function_calculate_topmsg(int direction);
 extern int owl_function_calculate_topmsg_top(int direction, owl_view *v, int curmsg, int topmsg, int recwinlines);
 extern int owl_function_calculate_topmsg_neartop(int direction, owl_view *v, int curmsg, int topmsg, int recwinlines);
@@ -456,7 +456,7 @@ extern char *owl_function_create_negative_filter(char *filtername);
 extern void owl_function_show_filters();
 extern void owl_function_show_filter(char *name);
 extern void owl_function_show_zpunts();
-extern char *owl_function_classinstfilt(char *class, char *instance);
+extern char *owl_function_classinstfilt(char *c, char *i);
 extern char *owl_function_zuserfilt(char *user);
 extern char *owl_function_aimuserfilt(char *user);
 extern char *owl_function_typefilt(char *type);
@@ -467,6 +467,7 @@ extern void owl_function_color_current_filter(char *color);
 extern int owl_function_color_filter(char *filtname, char *color);
 extern void owl_function_show_colors();
 extern void owl_function_zpunt(char *class, char *inst, char *recip, int direction);
+extern void owl_function_punt(char *filter, int direction);
 extern void owl_function_activate_keymap(char *keymap);
 extern void owl_function_show_keymaps();
 extern char *owl_function_keymap_summary(void *name);
@@ -548,7 +549,7 @@ extern void owl_global_resize(owl_global *g, int x, int y);
 extern int owl_global_is_debug_fast(owl_global *g);
 extern time_t owl_global_get_starttime(owl_global *g);
 extern time_t owl_global_get_runtime(owl_global *g);
-extern void owl_global_get_runtime_string(owl_global *g, char *buff);
+extern char *owl_global_get_runtime_string(owl_global *g);
 extern char *owl_global_get_hostname(owl_global *g);
 extern void owl_global_set_userclue(owl_global *g, int clue);
 extern void owl_global_add_userclue(owl_global *g, int clue);
@@ -561,7 +562,6 @@ extern void owl_global_set_startupargs(owl_global *g, int argc, char **argv);
 extern char *owl_global_get_startupargs(owl_global *g);
 extern owl_history *owl_global_get_msg_history(owl_global *g);
 extern owl_history *owl_global_get_cmd_history(owl_global *g);
-extern owl_muxevents *owl_global_get_muxevents(owl_global *g);
 extern owl_list *owl_global_get_filterlist(owl_global *g);
 extern owl_filter *owl_global_get_filter(owl_global *g, char *name);
 extern void owl_global_add_filter(owl_global *g, owl_filter *f);
@@ -620,6 +620,7 @@ extern int owl_global_get_errsignal_and_clear(owl_global *g, siginfo_t *siginfo)
 extern owl_timer *owl_global_get_zephyr_buddycheck_timer(owl_global *g);
 extern owl_zbuddylist *owl_global_get_zephyr_buddylist(owl_global *g);
 extern struct termios *owl_global_get_startup_tio(owl_global *g);
+extern owl_list *owl_global_get_dispatchlist(owl_global *g);
 
 /* -------------------------------- help.c -------------------------------- */
 extern void owl_help();
@@ -805,19 +806,13 @@ extern void owl_message_free(owl_message *m);
 extern int owl_messagelist_create(owl_messagelist *ml);
 extern int owl_messagelist_get_size(owl_messagelist *ml);
 extern void *owl_messagelist_get_element(owl_messagelist *ml, int n);
-extern owl_message *owl_messagelist_get_by_id(owl_messagelist *ml, int id);
+extern owl_message *owl_messagelist_get_by_id(owl_messagelist *ml, int target_id);
 extern int owl_messagelist_append_element(owl_messagelist *ml, void *element);
 extern int owl_messagelist_delete_element(owl_messagelist *ml, int n);
 extern int owl_messagelist_undelete_element(owl_messagelist *ml, int n);
 extern int owl_messagelist_expunge(owl_messagelist *ml);
 extern void owl_messagelist_invalidate_formats(owl_messagelist *ml);
 
-/* -------------------------------- muxevents.c -------------------------------- */
-extern int owl_muxevents_add(owl_muxevents *muxevents, int fd, int eventmask, void (*handler_fn)(int handle, int fd, int eventmask, void *data), void *data);
-extern void owl_muxevents_remove(owl_muxevents *muxevents, int handle);
-extern void owl_muxevents_gc(owl_muxevents *muxevents);
-extern void owl_muxevents_dispatch(owl_muxevents *muxevents, int timeout_usec);
-
 /* -------------------------------- owl.c -------------------------------- */
 #if OWL_STDERR_REDIR
 #ifdef HAVE_SYS_IOCTL_H
@@ -828,10 +823,6 @@ extern void owl_muxevents_dispatch(owl_muxevents *muxevents, int timeout_usec);
 extern int main(int argc, char **argv, char **env);
 #ifdef HAVE_LIBZEPHYR
 #endif
-#if OWL_STDERR_REDIR
-#endif
-#ifdef HAVE_LIBZEPHYR
-#endif
 #ifdef HAVE_USE_DEFAULT_COLORS
 #endif
 #ifdef HAVE_LIBZEPHYR
@@ -840,15 +831,12 @@ extern int main(int argc, char **argv, char **env);
 #endif   
 #if 0
 #endif
-#ifdef HAVE_LIBZEPHYR
-#endif
-#ifdef HAVE_LIBZEPHYR
-#endif
+extern void owl_process_input(owl_dispatch *d);
 extern void sig_handler(int sig, siginfo_t *si, void *data);
 extern void usage();
 #if OWL_STDERR_REDIR
 extern int stderr_replace(void);
-extern void stderr_redirect_handler(int handle, int rfd, int eventmask, void *data);
+extern void stderr_redirect_handler(owl_dispatch *d);
 #endif /* OWL_STDERR_REDIR */
 
 /* -------------------------------- pair.c -------------------------------- */
@@ -880,7 +868,8 @@ extern char *owl_perlconfig_getmsg(owl_message *m, int mode, char *subname);
 #ifdef HAVE_SYS_FILIO_H
 #endif
 extern owl_popexec *owl_popexec_new(char *command);
-extern void owl_popexec_inputhandler(int handle, int fd, int eventmask, void *data);
+extern void owl_popexec_inputhandler(owl_dispatch *d);
+extern void owl_popexec_free_dispatch(owl_dispatch *d);
 extern void owl_popexec_viewwin_onclose(owl_viewwin *vwin, void *data);
 extern void owl_popexec_unref(owl_popexec *pe);
 
@@ -909,6 +898,17 @@ extern char *owl_regex_get_string(owl_regex *re);
 extern void owl_regex_copy(owl_regex *a, owl_regex *b);
 extern void owl_regex_free(owl_regex *re);
 
+/* -------------------------------- select.c -------------------------------- */
+extern int owl_select_find_dispatch(int fd);
+extern void owl_select_add_dispatch(owl_dispatch *d);
+extern void owl_select_remove_dispatch(int fd);
+extern int owl_select_dispatch_count();
+extern int owl_select_dispatch_prepare_fd_sets(fd_set *r, fd_set *e);
+extern void owl_select_gc();
+extern void owl_select_dispatch(fd_set *fds, int max_fd);
+extern int owl_select_aim_hack(fd_set *rfds, fd_set *wfds);
+extern void owl_select();
+
 /* -------------------------------- style.c -------------------------------- */
 extern void owl_style_create_internal(owl_style *s, char *name, void (*formatfunc) (owl_fmtext *fm, owl_message *m), char *description);
 extern void owl_style_create_perl(owl_style *s, char *name, char *perlfuncname, char *description);
@@ -1002,6 +1002,7 @@ extern void owl_util_list_add_unique_string(owl_list *list, char *str);
 extern int owl_util_common_strings_in_lists(owl_list *a, owl_list *b);
 extern int owl_util_max(int a, int b);
 extern int owl_util_min(int a, int b);
+extern char * owl_util_baseclass(char * class);
 #ifdef OWL_INCLUDE_REG_TESTS
 extern int owl_util_regtest(void);
 #endif /* OWL_INCLUDE_REG_TESTS */
@@ -1178,8 +1179,6 @@ extern void owl_global_set_typwin_lines(owl_global *g, int n);
 extern int owl_global_get_typwin_lines(owl_global *g);
 extern void owl_global_set_scrollmode(owl_global *g, int n);
 extern int owl_global_get_scrollmode(owl_global *g);
-extern void owl_global_set_webbrowser(owl_global *g, int n);
-extern int owl_global_get_webbrowser(owl_global *g);
 extern void owl_global_set__followlast_on(owl_global *g);
 extern void owl_global_set__followlast_off(owl_global *g);
 extern int owl_global_is__followlast(owl_global *g);
@@ -1269,6 +1268,9 @@ extern char *owl_zephyr_get_sender();
 #ifdef HAVE_LIBZEPHYR
 #else
 #endif
+#ifdef HAVE_LIBZEPHYR
+extern int owl_zephyr_loadsubs_helper(ZSubscription_t subs[], int count);
+#endif
 extern int owl_zephyr_loadsubs(char *filename, int error_on_nofile);
 #ifdef HAVE_LIBZEPHYR
 #else
@@ -1336,8 +1338,9 @@ extern void owl_zephyr_zaway(owl_message *m);
 #ifdef HAVE_LIBZEPHYR
 extern void owl_zephyr_hackaway_cr(ZNotice_t *n);
 #endif
-extern void owl_zephyr_zlocate(char *user, char *out, int auth);
+extern char *owl_zephyr_zlocate(char *user, int auth);
 #ifdef HAVE_LIBZEPHYR
+#else
 #endif
 extern void owl_zephyr_addsub(char *filename, char *class, char *inst, char *recip);
 #ifdef HAVE_LIBZEPHYR
@@ -1377,6 +1380,11 @@ extern int owl_zephyr_get_anyone_list(owl_list *in, char *filename);
 #ifdef HAVE_LIBZEPHYR
 #else
 #endif
+#ifdef HAVE_LIBZEPHYR
+extern void owl_zephyr_process_events(owl_dispatch *d);
+#else
+extern void owl_zephyr_process_events(owl_dispatch *d);
+#endif
 
 /* -------------------------------- zwrite.c -------------------------------- */
 extern int owl_zwrite_create_from_line(owl_zwrite *z, char *line);
@@ -1394,7 +1402,6 @@ extern char *owl_zwrite_get_opcode(owl_zwrite *z);
 extern void owl_zwrite_set_opcode(owl_zwrite *z, char *opcode);
 extern char *owl_zwrite_get_realm(owl_zwrite *z);
 extern char *owl_zwrite_get_zsig(owl_zwrite *z);
-extern void owl_zwrite_get_recipstr(owl_zwrite *z, char *buff);
 extern int owl_zwrite_get_numrecips(owl_zwrite *z);
 extern char *owl_zwrite_get_recip_n(owl_zwrite *z, int n);
 extern int owl_zwrite_is_personal(owl_zwrite *z);
