@@ -1,6 +1,30 @@
+/* Copyright (c) 2002,2003,2004,2009 James M. Kretchmar
+ *
+ * This file is part of Owl.
+ *
+ * Owl is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Owl is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Owl.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * ---------------------------------------------------------------
+ * 
+ * As of Owl version 2.1.12 there are patches contributed by
+ * developers of the branched BarnOwl project, Copyright (c)
+ * 2006-2009 The BarnOwl Developers. All rights reserved.
+ */
+
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: stylefunc.c,v 1.19 2004/04/11 00:01:57 kretch Exp $";
+static const char fileIdent[] = "$Id: stylefunc.c,v 1.22 2009/04/05 23:36:54 kretch Exp $";
 
 /* In all of these functions, 'fm' is expected to already be
  * initialized.
@@ -9,7 +33,7 @@ static const char fileIdent[] = "$Id: stylefunc.c,v 1.19 2004/04/11 00:01:57 kre
 void owl_stylefunc_basic(owl_fmtext *fm, owl_message *m)
 {
 #ifdef HAVE_LIBZEPHYR
-  char *body, *indent, *ptr, *zsigbuff, frombuff[LINE];
+  char *body, *indent, *ptr, *zsigbuff, *frombuff;
   ZNotice_t *n;
 #endif
 
@@ -31,11 +55,7 @@ void owl_stylefunc_basic(owl_fmtext *fm, owl_message *m)
     owl_text_indent(indent, body, OWL_MSGTAB);
     
     /* edit the from addr for printing */
-    strcpy(frombuff, owl_message_get_sender(m));
-    ptr=strchr(frombuff, '@');
-    if (ptr && !strncmp(ptr+1, owl_zephyr_get_realm(), strlen(owl_zephyr_get_realm()))) {
-      *ptr='\0';
-    }
+    frombuff=short_zuser(owl_message_get_sender(m));
     
     /* set the message for printing */
     owl_fmtext_append_normal(fm, OWL_TABSTR);
@@ -106,6 +126,7 @@ void owl_stylefunc_basic(owl_fmtext *fm, owl_message *m)
     
     owl_free(body);
     owl_free(indent);
+    owl_free(frombuff);
 #endif
   } else if (owl_message_is_type_zephyr(m) && owl_message_is_direction_out(m)) {
     char *indent, *text, *zsigbuff, *foo;
@@ -218,7 +239,7 @@ void owl_stylefunc_default(owl_fmtext *fm, owl_message *m)
 {
   char *shorttimestr;
 #ifdef HAVE_LIBZEPHYR
-  char *body, *indent, *ptr, *zsigbuff, frombuff[LINE];
+  char *body, *indent, *ptr, *zsigbuff, *frombuff;
   ZNotice_t *n;
 #endif
 
@@ -242,11 +263,7 @@ void owl_stylefunc_default(owl_fmtext *fm, owl_message *m)
     owl_text_indent(indent, body, OWL_MSGTAB);
     
     /* edit the from addr for printing */
-    strcpy(frombuff, owl_message_get_sender(m));
-    ptr=strchr(frombuff, '@');
-    if (ptr && !strncmp(ptr+1, owl_zephyr_get_realm(), strlen(owl_zephyr_get_realm()))) {
-      *ptr='\0';
-    }
+    frombuff=short_zuser(owl_message_get_sender(m));
     
     /* set the message for printing */
     owl_fmtext_append_normal(fm, OWL_TABSTR);
@@ -323,6 +340,7 @@ void owl_stylefunc_default(owl_fmtext *fm, owl_message *m)
     
     owl_free(body);
     owl_free(indent);
+    owl_free(frombuff);
 #endif
   } else if (owl_message_is_type_zephyr(m) && owl_message_is_direction_out(m)) {
     char *indent, *text, *zsigbuff, *foo;
@@ -579,7 +597,7 @@ void owl_stylefunc_oneline(owl_fmtext *fm, owl_message *m)
 void owl_stylefunc_vt(owl_fmtext *fm, owl_message *m)
 {
 #ifdef HAVE_LIBZEPHYR
-  char *body, *indent, *ptr, frombuff[LINE];
+  char *body, *indent, *ptr, *frombuff;
   owl_fmtext fm_first, fm_other, fm_tmp;
   ZNotice_t *n;
 #endif
@@ -615,11 +633,7 @@ void owl_stylefunc_vt(owl_fmtext *fm, owl_message *m)
     owl_fmtext_free(&fm_tmp);
     
     /* edit the from addr for printing */
-    strcpy(frombuff, owl_message_get_sender(m));
-    ptr=strchr(frombuff, '@');
-    if (ptr && !strncmp(ptr+1, owl_zephyr_get_realm(), strlen(owl_zephyr_get_realm()))) {
-      *ptr='\0';
-    }
+    frombuff=short_zuser(owl_message_get_sender(m));
     sender=owl_sprintf("%-9.9s", frombuff);
 
     hostname=owl_sprintf("%-9.9s", owl_message_get_hostname(m));
@@ -682,6 +696,7 @@ void owl_stylefunc_vt(owl_fmtext *fm, owl_message *m)
     }
 
     owl_free(sender);
+    owl_free(frombuff);
     owl_free(hostname);
     owl_free(timestr);
     owl_free(classinst1);
