@@ -1,10 +1,34 @@
+/* Copyright (c) 2002,2003,2004,2009 James M. Kretchmar
+ *
+ * This file is part of Owl.
+ *
+ * Owl is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Owl is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Owl.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * ---------------------------------------------------------------
+ * 
+ * As of Owl version 2.1.12 there are patches contributed by
+ * developers of the branched BarnOwl project, Copyright (c)
+ * 2006-2009 The BarnOwl Developers. All rights reserved.
+ */
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: text.c,v 1.12 2004/03/18 05:04:40 nygren Exp $";
+static const char fileIdent[] = "$Id: text.c,v 1.15 2009/04/05 23:36:54 kretch Exp $";
 
 /* start with line aline (where the first line is 1) and print 'lines'
  *  lines
@@ -326,7 +350,7 @@ char *owl_text_substitute(char *in, char *from, char *to)
   outlen = strlen(in)+1;
   tolen  = strlen(to);
   fromlen  = strlen(from);
-  out = malloc(outlen);
+  out = owl_malloc(outlen);
 
   while (in[inpos]) {
     if (!strncmp(in+inpos, from, fromlen)) {
