@@ -1,3 +1,27 @@
+/* Copyright (c) 2002,2003,2004,2009 James M. Kretchmar
+ *
+ * This file is part of Owl.
+ *
+ * Owl is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Owl is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Owl.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * ---------------------------------------------------------------
+ * 
+ * As of Owl version 2.1.12 there are patches contributed by
+ * developers of the branched BarnOwl project, Copyright (c)
+ * 2006-2009 The BarnOwl Developers. All rights reserved.
+ */
+
 #include "owl.h"
 #include <stdlib.h>
 #include <string.h>
@@ -5,7 +29,7 @@
 #include <ctype.h>
 #include <pwd.h>
 
-static const char fileIdent[] = "$Id: util.c,v 1.46 2004/04/18 15:55:42 kretch Exp $";
+static const char fileIdent[] = "$Id: util.c,v 1.49 2009/03/29 19:08:14 kretch Exp $";
 
 void sepbar(char *in)
 {
@@ -345,6 +369,8 @@ char **owl_parseline(char *line, int *argc)
     curarg[strlen(curarg)]=line[i];
   }
 
+  owl_free(curarg);
+
   /* check for unbalanced quotes */
   if (quote!='\0') {
     owl_parsefree(argv, *argc);
@@ -647,9 +673,9 @@ void owl_util_file_deleteline(char *filename, char *line, int backup)
     if (!backupfile) {
       owl_function_error("Error opening file %s for writing", backupfilename);
       owl_free(backupfilename);
+      fclose(file);
       return;
     }
-    owl_free(backupfilename);
   }
 
   /* we'll read the entire file into memory, minus the line we don't want and
@@ -688,12 +714,14 @@ void owl_util_file_deleteline(char *filename, char *line, int backup)
   if (!file) {
     owl_function_error("WARNING: Error opening %s for writing.  Use %s to restore.", filename, backupfilename);
     owl_function_beep();
-    owl_free(line);
-    return;
+  } else {
+    fputs(text, file);
+    fclose(file);
   }
 
-  fputs(text, file);
-  fclose(file);
+  if (backup)
+    owl_free(backupfilename);
+  owl_free(text);
 }
 
 /* add the string 'str' to the list 'list' of strings, only if it
@@ -736,6 +764,29 @@ int owl_util_min(int a, int b)
   return(b);
 }
 
+/* Return the base class or instance from a zephyr class, by removing
+   leading `un' or trailing `.d'.
+   The caller is responsible for freeing the allocated string.
+*/
+char * owl_util_baseclass(char * class)
+{
+  char *start, *end;
+
+  start = class;
+  while(!strncmp(start, "un", 2)) {
+    start += 2;
+  }
+
+  start = owl_strdup(start);
+  end = start + strlen(start) - 1;
+  while(end > start && *end == 'd' && *(end-1) == '.') {
+    end -= 2;
+  }
+  *(end + 1) = 0;
+
+  return start;
+}
+
 /**************************************************************************/
 /************************* REGRESSION TESTS *******************************/
 /**************************************************************************/
