@@ -6,7 +6,7 @@
 #include <string.h>
 #include "owl.h"
 
-static const char fileIdent[] = "$Id: zephyr.c,v 1.52 2005/01/05 04:29:48 kretch Exp $";
+static const char fileIdent[] = "$Id: zephyr.c,v 1.61 2009/04/05 23:36:54 kretch Exp $";
 
 #ifdef HAVE_LIBZEPHYR
 Code_t ZResetAuthentication();
@@ -66,6 +66,26 @@ char *owl_zephyr_get_sender()
 #endif
 }
 
+#ifdef HAVE_LIBZEPHYR
+int owl_zephyr_loadsubs_helper(ZSubscription_t subs[], int count)
+{
+  int i, ret = 0;
+  /* sub without defaults */
+  if (ZSubscribeToSansDefaults(subs,count,0) != ZERR_NONE) {
+    owl_function_error("Error subscribing to zephyr notifications.");
+    ret=-2;
+  }
+
+  /* free stuff */
+  for (i=0; i<count; i++) {
+    owl_free(subs[i].zsub_class);
+    owl_free(subs[i].zsub_classinst);
+    owl_free(subs[i].zsub_recipient);
+  }
+  return ret;
+}
+#endif
+
 /* Load zephyr subscriptions form 'filename'.  If 'filename' is NULL,
  * the default file $HOME/.zephyr.subs will be used.
  *
@@ -79,9 +99,9 @@ int owl_zephyr_loadsubs(char *filename, int error_on_nofile)
 #ifdef HAVE_LIBZEPHYR
   FILE *file;
   char *tmp, *start;
-  char buffer[1024], subsfile[1024];
+  char buffer[1024], subsfile[MAXPATHLEN];
   ZSubscription_t subs[3001];
-  int count, ret, i;
+  int count, ret;
   struct stat statbuff;
 
   if (filename==NULL) {
@@ -92,15 +112,18 @@ int owl_zephyr_loadsubs(char *filename, int error_on_nofile)
 
   ret=stat(subsfile, &statbuff);
   if (ret) {
+    owl_function_debugmsg("loadsubs: could not stat file %s", subsfile);
     if (error_on_nofile==1) return(-1);
     return(0);
   }
 
   ZResetAuthentication();
-  /* need to redo this to do chunks, not just bail after 3000 */
   count=0;
   file=fopen(subsfile, "r");
-  if (!file) return(-1);
+  if (!file) {
+    owl_function_debugmsg("loadsubs: could not open file %s", subsfile);
+    return(-1);
+  }
   while ( fgets(buffer, 1024, file)!=NULL ) {
     if (buffer[0]=='#' || buffer[0]=='\n' || buffer[0]=='\n') continue;
     
@@ -110,7 +133,14 @@ int owl_zephyr_loadsubs(char *filename, int error_on_nofile)
       start=buffer;
     }
     
-    if (count >= 3000) break; /* also tell the user */
+    if (count >= 3000) {
+      ret = owl_zephyr_loadsubs_helper(subs, count);
+      if (ret != 0) {
+	fclose(file);
+	return(ret);
+      }
+      count=0;
+    }
     
     /* add it to the list of subs */
     if ((tmp=(char *) strtok(start, ",\n\r"))==NULL) continue;
@@ -129,19 +159,7 @@ int owl_zephyr_loadsubs(char *filename, int error_on_nofile)
   }
   fclose(file);
 
-  /* sub without defaults */
-  ret=0;
-  if (ZSubscribeToSansDefaults(subs,count,0) != ZERR_NONE) {
-    owl_function_error("Error subscribing to zephyr notifications.");
-    ret=-2;
-  }
-
-  /* free stuff */
-  for (i=0; i<count; i++) {
-    owl_free(subs[i].zsub_class);
-    owl_free(subs[i].zsub_classinst);
-    owl_free(subs[i].zsub_recipient);
-  }
+  ret=owl_zephyr_loadsubs_helper(subs, count);
 
   return(ret);
 #else
@@ -169,7 +187,7 @@ int owl_zephyr_loadloginsubs(char *filename)
 #ifdef HAVE_LIBZEPHYR
   FILE *file;
   ZSubscription_t subs[3001];
-  char subsfile[1024], buffer[1024];
+  char subsfile[MAXPATHLEN], buffer[1024];
   int count, ret, i;
   struct stat statbuff;
 
@@ -291,6 +309,10 @@ char *owl_zephyr_get_field(ZNotice_t *n, int j)
   int i, count, save;
   char *out;
 
+  /* If there's no message here, just run along now */
+  if (n->z_message_len == 0)
+    return(owl_strdup(""));
+
   count=save=0;
   for (i=0; i<n->z_message_len; i++) {
     if (n->z_message[i]=='\0') {
@@ -326,6 +348,9 @@ int owl_zephyr_get_num_fields(ZNotice_t *n)
 {
   int i, fields;
 
+  if(n->z_message_len == 0)
+    return 0;
+
   fields=1;
   for (i=0; i<n->z_message_len; i++) {
     if (n->z_message[i]=='\0') fields++;
@@ -355,6 +380,21 @@ char *owl_zephyr_get_message(ZNotice_t *n)
   if (!strcasecmp(n->z_sender, "olc.matisse@ATHENA.MIT.EDU")) {
     return(owl_zephyr_get_field(n, 1));
   }
+  /* deal with MIT NOC messages */
+  else if (!strcasecmp(n->z_sender, "rcmd.achilles@ATHENA.MIT.EDU")) {
+    /* $opcode service on $instance $3.\n$4 */
+    char *msg, *opcode, *instance, *field3, *field4;
+
+    opcode = n->z_opcode;
+    instance = n->z_class_inst;
+    field3 = owl_zephyr_get_field(n, 3);
+    field4 = owl_zephyr_get_field(n, 4);
+
+    msg = owl_sprintf("%s service on %s %s\n%s", opcode, instance, field3, field4);
+    if (msg) {
+      return msg;
+    }
+  }
 
   return(owl_zephyr_get_field(n, 2));
 }
@@ -436,7 +476,9 @@ int send_zephyr(char *opcode, char *zsig, char *class, char *instance, char *rec
 #ifdef HAVE_LIBZEPHYR
 Code_t send_zephyr_helper(ZNotice_t *notice, char *buf, int len, int wait)
 {
-  return(ZSendPacket(buf, len, 0));
+  int ret;
+  ret=ZSendPacket(buf, len, 0);
+  return(ret);
 }
 #endif
 
@@ -471,16 +513,31 @@ void owl_zephyr_handle_ack(ZNotice_t *retnotice)
       owl_function_makemsg("Message sent to -c %s -i %s\n", retnotice->z_class, retnotice->z_class_inst);
     }
   } else if (!strcmp(retnotice->z_message, ZSRVACK_NOTSENT)) {
-    if (strcasecmp(retnotice->z_class, "message")) {
-      char buff[1024];
-      owl_function_error("No one subscribed to class class %s", retnotice->z_class);
-      sprintf(buff, "Could not send message to class %s: no one subscribed.\n", retnotice->z_class);
+    #define BUFFLEN 1024
+    if (retnotice->z_recipient == NULL
+        || *retnotice->z_recipient == 0
+        || *retnotice->z_recipient == '@') {
+      char buff[BUFFLEN];
+      owl_function_error("No one subscribed to class %s", retnotice->z_class);
+      snprintf(buff, BUFFLEN, "Could not send message to class %s: no one subscribed.\n", retnotice->z_class);
       owl_function_adminmsg("", buff);
     } else {
-      char buff[1024];
+      char buff[BUFFLEN];
       tmp = short_zuser(retnotice->z_recipient);
-      owl_function_error("%s: Not logged in or subscribing to messages.", tmp);
-      sprintf(buff, "Could not send message to %s: not logged in or subscribing to messages.\n", tmp);
+      owl_function_error("%s: Not logged in or subscribing.", tmp);
+      if(strcmp(retnotice->z_class, "message")) {
+        snprintf(buff, BUFFLEN,
+                 "Could not send message to %s: "
+                 "not logged in or subscribing to class %s, instance %s.\n", 
+                 tmp,
+                 retnotice->z_class,
+                 retnotice->z_class_inst);
+      } else {
+        snprintf(buff, BUFFLEN,
+                 "Could not send message to %s: "
+                 "not logged in or subscribing to messages.\n",
+                 tmp);
+      }
       owl_function_adminmsg("", buff);
       owl_log_outgoing_zephyr_error(tmp, buff);
       owl_free(tmp);
@@ -572,45 +629,51 @@ void owl_zephyr_hackaway_cr(ZNotice_t *n)
 }
 #endif
 
-void owl_zephyr_zlocate(char *user, char *out, int auth)
+char *owl_zephyr_zlocate(char *user, int auth)
 {
 #ifdef HAVE_LIBZEPHYR
   int ret, numlocs;
   int one = 1;
   ZLocations_t locations;
-  char *myuser;
+  char *myuser, *out, *tmp;
   
-  strcpy(out, "");
   ZResetAuthentication();
   ret=ZLocateUser(user,&numlocs,auth?ZAUTH:ZNOAUTH);
   if (ret != ZERR_NONE) {
-    sprintf(out, "Error locating user %s\n", user);
-    return;
+    return(owl_sprintf("Error locating user %s\n", user));
   }
 
   if (numlocs==0) {
     myuser=short_zuser(user);
-    sprintf(out, "%s: Hidden or not logged-in\n", myuser);
+    out=owl_sprintf("%s: Hidden or not logged in\n", myuser);
     owl_free(myuser);
-    return;
+    return(out);
   }
-    
+
+  out=strdup("");
   for (;numlocs;numlocs--) {
     ZGetLocations(&locations,&one);
     myuser=short_zuser(user);
-    sprintf(out, "%s%s: %s\t%s\t%s\n", out, myuser,
-	    locations.host ? locations.host : "?",
-	    locations.tty ? locations.tty : "?",
-	    locations.time ? locations.time : "?");
+    tmp=owl_sprintf("%s%s: %s\t%s\t%s\n",
+		    out,
+		    myuser,
+		    locations.host ? locations.host : "?",
+		    locations.tty ? locations.tty : "?",
+		    locations.time ? locations.time : "?");
+    owl_free(out);
+    out=tmp;
     owl_free(myuser);
   }
+  return(out);
+#else
+  return(owl_strdup("Zephyr not available"));
 #endif
 }
 
 void owl_zephyr_addsub(char *filename, char *class, char *inst, char *recip)
 {
 #ifdef HAVE_LIBZEPHYR
-  char *line, subsfile[LINE], buff[LINE];
+  char *line, subsfile[MAXPATHLEN], buff[LINE];
   FILE *file;
 
   line=owl_zephyr_makesubline(class, inst, recip);
@@ -628,13 +691,14 @@ void owl_zephyr_addsub(char *filename, char *class, char *inst, char *recip)
       if (!strcasecmp(buff, line)) {
 	owl_function_error("Subscription already present in %s", subsfile);
 	owl_free(line);
+	fclose(file);
 	return;
       }
     }
+    fclose(file);
   }
 
   /* if we get here then we didn't find it */
-  fclose(file);
   file=fopen(subsfile, "a");
   if (!file) {
     owl_function_error("Error opening file %s for writing", subsfile);
@@ -673,14 +737,9 @@ void owl_zephyr_delsub(char *filename, char *class, char *inst, char *recip)
 /* caller must free the return */
 char *owl_zephyr_makesubline(char *class, char *inst, char *recip)
 {
-  char *out;
-
-  out=owl_malloc(strlen(class)+strlen(inst)+strlen(recip)+30);
-  sprintf(out, "%s,%s,%s\n", class, inst, !strcmp(recip, "") ? "*" : recip);
-  return(out);
+  return(owl_sprintf("%s,%s,%s\n", class, inst, !strcmp(recip, "") ? "*" : recip));
 }
 
-
 void owl_zephyr_zlog_in(void)
 {
 #ifdef HAVE_LIBZEPHYR
@@ -708,12 +767,9 @@ void owl_zephyr_zlog_in(void)
   }
    
   ret=ZSetLocation(eset);
+  owl_zephyr_process_events(NULL);
   if (ret != ZERR_NONE) {
-    /*
-      char buff[LINE];
-      sprintf(buff, "Error setting location: %s", error_message(ret));
-      owl_function_makemsg(buff);
-    */
+    /* owl_function_makemsg("Error setting location: %s", error_message(ret)); */
   }
 #endif
 }
@@ -725,12 +781,9 @@ void owl_zephyr_zlog_out(void)
 
   ZResetAuthentication();
   ret=ZUnsetLocation();
+  owl_zephyr_process_events(NULL);
   if (ret != ZERR_NONE) {
-    /*
-      char buff[LINE];
-      sprintf(buff, "Error unsetting location: %s", error_message(ret));
-      owl_function_makemsg(buff);
-    */
+    /* owl_function_makemsg("Error unsetting location: %s", error_message(ret)); */
   }
 #endif
 }
@@ -791,34 +844,50 @@ char *owl_zephyr_getsubs()
 {
 #ifdef HAVE_LIBZEPHYR
   int ret, num, i, one;
+  int buffsize;
   ZSubscription_t sub;
-  char *out, *tmpbuff;
+  char *out;
   one=1;
 
   ret=ZRetrieveSubscriptions(0, &num);
   if (ret==ZERR_TOOMANYSUBS) {
     return(owl_strdup("Zephyr: too many subscriptions\n"));
-  } else if (ret) {
+  } else if (ret || (num <= 0)) {
     return(owl_strdup("Zephyr: error retriving subscriptions\n"));
   }
 
-  out=owl_malloc(num*500);
-  tmpbuff=owl_malloc(num*500);
+  buffsize = (num + 1) * 50;
+  out=owl_malloc(buffsize);
   strcpy(out, "");
   for (i=0; i<num; i++) {
     if ((ret = ZGetSubscriptions(&sub, &one)) != ZERR_NONE) {
       owl_free(out);
-      owl_free(tmpbuff);
       ZFlushSubscriptions();
       out=owl_strdup("Error while getting subscriptions\n");
       return(out);
     } else {
-      sprintf(tmpbuff, "<%s,%s,%s>\n%s", sub.zsub_class, sub.zsub_classinst, sub.zsub_recipient, out);
+      int tmpbufflen;
+      char *tmpbuff;
+      tmpbuff = owl_sprintf("<%s,%s,%s>\n%s", sub.zsub_class, sub.zsub_classinst, sub.zsub_recipient, out);
+      tmpbufflen = strlen(tmpbuff) + 1;
+      if (tmpbufflen > buffsize) {
+        char *out2;
+        buffsize = tmpbufflen * 2;
+        out2 = owl_realloc(out, buffsize);
+        if (out2 == NULL) {
+          owl_free(out);
+          owl_free(tmpbuff);
+          ZFlushSubscriptions();
+          out=owl_strdup("Realloc error while getting subscriptions\n");
+          return(out);    
+        }
+        out = out2;
+      }
       strcpy(out, tmpbuff);
+      owl_free(tmpbuff);
     }
   }
 
-  owl_free(tmpbuff);
   ZFlushSubscriptions();
   return(out);
 #else
@@ -968,3 +1037,42 @@ int owl_zephyr_get_anyone_list(owl_list *in, char *filename)
   return(-1);
 #endif
 }
+
+
+#ifdef HAVE_LIBZEPHYR
+void owl_zephyr_process_events(owl_dispatch *d) {
+  int zpendcount=0;
+  ZNotice_t notice;
+  struct sockaddr_in from;
+  owl_message *m=NULL;
+
+  while(owl_zephyr_zpending() && zpendcount < 20) {
+    if (owl_zephyr_zpending()) {
+      ZReceiveNotice(&notice, &from);
+      zpendcount++;
+
+      /* is this an ack from a zephyr we sent? */
+      if (owl_zephyr_notice_is_ack(&notice)) {
+        owl_zephyr_handle_ack(&notice);
+        continue;
+      }
+
+      /* if it's a ping and we're not viewing pings then skip it */
+      if (!owl_global_is_rxping(&g) && !strcasecmp(notice.z_opcode, "ping")) {
+        continue;
+      }
+
+      /* create the new message */
+      m=owl_malloc(sizeof(owl_message));
+      owl_message_create_from_znotice(m, &notice);
+
+      owl_global_messagequeue_addmsg(&g, m);
+    }
+  }
+}
+
+#else
+void owl_zephyr_process_events(owl_dispatch *d) {
+  
+}
+#endif
